@@ -16,12 +16,14 @@ static t_class *plus_class, *scalarplus_class;
 typedef struct _plus
 {
     t_object x_obj;
+    float x_f;
 } t_plus;
 
 typedef struct _scalarplus
 {
     t_object x_obj;
-    t_float x_f;
+    float x_f;
+    t_float x_g;    	    /* inlet value */
 } t_scalarplus;
 
 static void *plus_new(t_symbol *s, int argc, t_atom *argv)
@@ -30,9 +32,10 @@ static void *plus_new(t_symbol *s, int argc, t_atom *argv)
     if (argc) 
     {
 	t_scalarplus *x = (t_scalarplus *)pd_new(scalarplus_class);
-	floatinlet_new(&x->x_obj, &x->x_f);
-	x->x_f = atom_getfloatarg(0, argc, argv);
+	floatinlet_new(&x->x_obj, &x->x_g);
+	x->x_g = atom_getfloatarg(0, argc, argv);
 	outlet_new(&x->x_obj, &s_signal);
+    	x->x_f = 0;
 	return (x);
     }
     else
@@ -40,6 +43,7 @@ static void *plus_new(t_symbol *s, int argc, t_atom *argv)
 	t_plus *x = (t_plus *)pd_new(plus_class);
 	inlet_new(&x->x_obj, &x->x_obj.ob_pd, &s_signal, &s_signal);
 	outlet_new(&x->x_obj, &s_signal);
+    	x->x_f = 0;
 	return (x);
     }
 }
@@ -117,10 +121,10 @@ static void plus_dsp(t_plus *x, t_signal **sp)
 static void scalarplus_dsp(t_scalarplus *x, t_signal **sp)
 {
     if (sp[0]->s_n&7)
-    	dsp_add(scalarplus_perform, 4, sp[0]->s_vec, &x->x_f,
+    	dsp_add(scalarplus_perform, 4, sp[0]->s_vec, &x->x_g,
 	    sp[1]->s_vec, sp[0]->s_n);
     else	
-    	dsp_add(scalarplus_perf8, 4, sp[0]->s_vec, &x->x_f,
+    	dsp_add(scalarplus_perf8, 4, sp[0]->s_vec, &x->x_g,
 	    sp[1]->s_vec, sp[0]->s_n);
 }
 
@@ -128,12 +132,12 @@ static void plus_setup(void)
 {
     plus_class = class_new(gensym("+~"), (t_newmethod)plus_new, 0,
     	sizeof(t_plus), 0, A_GIMME, 0);
-    class_addmethod(plus_class, nullfn, gensym("signal"), 0);
     class_addmethod(plus_class, (t_method)plus_dsp, gensym("dsp"), 0);
+    CLASS_MAINSIGNALIN(plus_class, t_plus, x_f);
     class_sethelpsymbol(plus_class, gensym("sigbinops"));
     scalarplus_class = class_new(gensym("+~"), 0, 0,
     	sizeof(t_scalarplus), 0, 0);
-    class_addmethod(scalarplus_class, nullfn, gensym("signal"), 0);
+    CLASS_MAINSIGNALIN(scalarplus_class, t_scalarplus, x_f);
     class_addmethod(scalarplus_class, (t_method)scalarplus_dsp, gensym("dsp"),
     	0);
     class_sethelpsymbol(scalarplus_class, gensym("sigbinops"));
@@ -145,12 +149,14 @@ static t_class *minus_class, *scalarminus_class;
 typedef struct _minus
 {
     t_object x_obj;
+    float x_f;
 } t_minus;
 
 typedef struct _scalarminus
 {
     t_object x_obj;
-    t_float x_f;
+    float x_f;
+    t_float x_g;
 } t_scalarminus;
 
 static void *minus_new(t_symbol *s, int argc, t_atom *argv)
@@ -159,9 +165,10 @@ static void *minus_new(t_symbol *s, int argc, t_atom *argv)
     if (argc) 
     {
 	t_scalarminus *x = (t_scalarminus *)pd_new(scalarminus_class);
-	floatinlet_new(&x->x_obj, &x->x_f);
-	x->x_f = atom_getfloatarg(0, argc, argv);
+	floatinlet_new(&x->x_obj, &x->x_g);
+	x->x_g = atom_getfloatarg(0, argc, argv);
 	outlet_new(&x->x_obj, &s_signal);
+    	x->x_f = 0;
 	return (x);
     }
     else
@@ -169,6 +176,7 @@ static void *minus_new(t_symbol *s, int argc, t_atom *argv)
 	t_minus *x = (t_minus *)pd_new(minus_class);
 	inlet_new(&x->x_obj, &x->x_obj.ob_pd, &s_signal, &s_signal);
 	outlet_new(&x->x_obj, &s_signal);
+    	x->x_f = 0;
 	return (x);
     }
 }
@@ -243,10 +251,10 @@ static void minus_dsp(t_minus *x, t_signal **sp)
 static void scalarminus_dsp(t_scalarminus *x, t_signal **sp)
 {
     if (sp[0]->s_n&7)
-    	dsp_add(scalarminus_perform, 4, sp[0]->s_vec, &x->x_f,
+    	dsp_add(scalarminus_perform, 4, sp[0]->s_vec, &x->x_g,
 	    sp[1]->s_vec, sp[0]->s_n);
     else	
-    	dsp_add(scalarminus_perf8, 4, sp[0]->s_vec, &x->x_f,
+    	dsp_add(scalarminus_perf8, 4, sp[0]->s_vec, &x->x_g,
 	    sp[1]->s_vec, sp[0]->s_n);
 }
 
@@ -254,12 +262,12 @@ static void minus_setup(void)
 {
     minus_class = class_new(gensym("-~"), (t_newmethod)minus_new, 0,
     	sizeof(t_minus), 0, A_GIMME, 0);
-    class_addmethod(minus_class, nullfn, gensym("signal"), 0);
+    CLASS_MAINSIGNALIN(minus_class, t_minus, x_f);
     class_addmethod(minus_class, (t_method)minus_dsp, gensym("dsp"), 0);
     class_sethelpsymbol(minus_class, gensym("sigbinops"));
     scalarminus_class = class_new(gensym("-~"), 0, 0,
     	sizeof(t_scalarminus), 0, 0);
-    class_addmethod(scalarminus_class, nullfn, gensym("signal"), 0);
+    CLASS_MAINSIGNALIN(scalarminus_class, t_scalarminus, x_f);
     class_addmethod(scalarminus_class, (t_method)scalarminus_dsp, gensym("dsp"),
     	0);
     class_sethelpsymbol(scalarminus_class, gensym("sigbinops"));
@@ -272,12 +280,14 @@ static t_class *times_class, *scalartimes_class;
 typedef struct _times
 {
     t_object x_obj;
+    float x_f;
 } t_times;
 
 typedef struct _scalartimes
 {
     t_object x_obj;
-    t_float x_f;
+    float x_f;
+    t_float x_g;
 } t_scalartimes;
 
 static void *times_new(t_symbol *s, int argc, t_atom *argv)
@@ -286,9 +296,10 @@ static void *times_new(t_symbol *s, int argc, t_atom *argv)
     if (argc) 
     {
 	t_scalartimes *x = (t_scalartimes *)pd_new(scalartimes_class);
-	floatinlet_new(&x->x_obj, &x->x_f);
-	x->x_f = atom_getfloatarg(0, argc, argv);
+	floatinlet_new(&x->x_obj, &x->x_g);
+	x->x_g = atom_getfloatarg(0, argc, argv);
 	outlet_new(&x->x_obj, &s_signal);
+    	x->x_f = 0;
 	return (x);
     }
     else
@@ -296,6 +307,7 @@ static void *times_new(t_symbol *s, int argc, t_atom *argv)
 	t_times *x = (t_times *)pd_new(times_class);
 	inlet_new(&x->x_obj, &x->x_obj.ob_pd, &s_signal, &s_signal);
 	outlet_new(&x->x_obj, &s_signal);
+    	x->x_f = 0;
 	return (x);
     }
 }
@@ -370,10 +382,10 @@ static void times_dsp(t_times *x, t_signal **sp)
 static void scalartimes_dsp(t_scalartimes *x, t_signal **sp)
 {
     if (sp[0]->s_n&7)
-    	dsp_add(scalartimes_perform, 4, sp[0]->s_vec, &x->x_f,
+    	dsp_add(scalartimes_perform, 4, sp[0]->s_vec, &x->x_g,
 	    sp[1]->s_vec, sp[0]->s_n);
     else	
-    	dsp_add(scalartimes_perf8, 4, sp[0]->s_vec, &x->x_f,
+    	dsp_add(scalartimes_perf8, 4, sp[0]->s_vec, &x->x_g,
 	    sp[1]->s_vec, sp[0]->s_n);
 }
 
@@ -381,12 +393,12 @@ static void times_setup(void)
 {
     times_class = class_new(gensym("*~"), (t_newmethod)times_new, 0,
     	sizeof(t_times), 0, A_GIMME, 0);
-    class_addmethod(times_class, nullfn, gensym("signal"), 0);
+    CLASS_MAINSIGNALIN(times_class, t_times, x_f);
     class_addmethod(times_class, (t_method)times_dsp, gensym("dsp"), 0);
     class_sethelpsymbol(times_class, gensym("sigbinops"));
     scalartimes_class = class_new(gensym("*~"), 0, 0,
     	sizeof(t_scalartimes), 0, 0);
-    class_addmethod(scalartimes_class, nullfn, gensym("signal"), 0);
+    CLASS_MAINSIGNALIN(scalartimes_class, t_scalartimes, x_f);
     class_addmethod(scalartimes_class, (t_method)scalartimes_dsp, gensym("dsp"),
     	0);
     class_sethelpsymbol(scalartimes_class, gensym("sigbinops"));
@@ -398,12 +410,14 @@ static t_class *over_class, *scalarover_class;
 typedef struct _over
 {
     t_object x_obj;
+    float x_f;
 } t_over;
 
 typedef struct _scalarover
 {
     t_object x_obj;
-    t_float x_f;
+    float x_f;
+    t_float x_g;
 } t_scalarover;
 
 static void *over_new(t_symbol *s, int argc, t_atom *argv)
@@ -412,9 +426,10 @@ static void *over_new(t_symbol *s, int argc, t_atom *argv)
     if (argc) 
     {
 	t_scalarover *x = (t_scalarover *)pd_new(scalarover_class);
-	floatinlet_new(&x->x_obj, &x->x_f);
-	x->x_f = atom_getfloatarg(0, argc, argv);
+	floatinlet_new(&x->x_obj, &x->x_g);
+	x->x_g = atom_getfloatarg(0, argc, argv);
 	outlet_new(&x->x_obj, &s_signal);
+    	x->x_f = 0;
 	return (x);
     }
     else
@@ -422,6 +437,7 @@ static void *over_new(t_symbol *s, int argc, t_atom *argv)
 	t_over *x = (t_over *)pd_new(over_class);
 	inlet_new(&x->x_obj, &x->x_obj.ob_pd, &s_signal, &s_signal);
 	outlet_new(&x->x_obj, &s_signal);
+    	x->x_f = 0;
 	return (x);
     }
 }
@@ -507,10 +523,10 @@ static void over_dsp(t_over *x, t_signal **sp)
 static void scalarover_dsp(t_scalarover *x, t_signal **sp)
 {
     if (sp[0]->s_n&7)
-    	dsp_add(scalarover_perform, 4, sp[0]->s_vec, &x->x_f,
+    	dsp_add(scalarover_perform, 4, sp[0]->s_vec, &x->x_g,
 	    sp[1]->s_vec, sp[0]->s_n);
     else	
-    	dsp_add(scalarover_perf8, 4, sp[0]->s_vec, &x->x_f,
+    	dsp_add(scalarover_perf8, 4, sp[0]->s_vec, &x->x_g,
 	    sp[1]->s_vec, sp[0]->s_n);
 }
 
@@ -518,12 +534,12 @@ static void over_setup(void)
 {
     over_class = class_new(gensym("/~"), (t_newmethod)over_new, 0,
     	sizeof(t_over), 0, A_GIMME, 0);
-    class_addmethod(over_class, nullfn, gensym("signal"), 0);
+    CLASS_MAINSIGNALIN(over_class, t_over, x_f);
     class_addmethod(over_class, (t_method)over_dsp, gensym("dsp"), 0);
     class_sethelpsymbol(over_class, gensym("sigbinops"));
     scalarover_class = class_new(gensym("/~"), 0, 0,
     	sizeof(t_scalarover), 0, 0);
-    class_addmethod(scalarover_class, nullfn, gensym("signal"), 0);
+    CLASS_MAINSIGNALIN(scalarover_class, t_scalarover, x_f);
     class_addmethod(scalarover_class, (t_method)scalarover_dsp, gensym("dsp"),
     	0);
     class_sethelpsymbol(scalarover_class, gensym("sigbinops"));
@@ -535,12 +551,14 @@ static t_class *max_class, *scalarmax_class;
 typedef struct _max
 {
     t_object x_obj;
+    float x_f;
 } t_max;
 
 typedef struct _scalarmax
 {
     t_object x_obj;
-    t_float x_f;
+    float x_f;
+    t_float x_g;
 } t_scalarmax;
 
 static void *max_new(t_symbol *s, int argc, t_atom *argv)
@@ -549,9 +567,10 @@ static void *max_new(t_symbol *s, int argc, t_atom *argv)
     if (argc) 
     {
 	t_scalarmax *x = (t_scalarmax *)pd_new(scalarmax_class);
-	floatinlet_new(&x->x_obj, &x->x_f);
-	x->x_f = atom_getfloatarg(0, argc, argv);
+	floatinlet_new(&x->x_obj, &x->x_g);
+	x->x_g = atom_getfloatarg(0, argc, argv);
 	outlet_new(&x->x_obj, &s_signal);
+    	x->x_f = 0;
 	return (x);
     }
     else
@@ -559,6 +578,7 @@ static void *max_new(t_symbol *s, int argc, t_atom *argv)
 	t_max *x = (t_max *)pd_new(max_class);
 	inlet_new(&x->x_obj, &x->x_obj.ob_pd, &s_signal, &s_signal);
 	outlet_new(&x->x_obj, &s_signal);
+    	x->x_f = 0;
 	return (x);
     }
 }
@@ -645,10 +665,10 @@ static void max_dsp(t_max *x, t_signal **sp)
 static void scalarmax_dsp(t_scalarmax *x, t_signal **sp)
 {
     if (sp[0]->s_n&7)
-    	dsp_add(scalarmax_perform, 4, sp[0]->s_vec, &x->x_f,
+    	dsp_add(scalarmax_perform, 4, sp[0]->s_vec, &x->x_g,
 	    sp[1]->s_vec, sp[0]->s_n);
     else	
-    	dsp_add(scalarmax_perf8, 4, sp[0]->s_vec, &x->x_f,
+    	dsp_add(scalarmax_perf8, 4, sp[0]->s_vec, &x->x_g,
 	    sp[1]->s_vec, sp[0]->s_n);
 }
 
@@ -656,12 +676,12 @@ static void max_setup(void)
 {
     max_class = class_new(gensym("max~"), (t_newmethod)max_new, 0,
     	sizeof(t_max), 0, A_GIMME, 0);
-    class_addmethod(max_class, nullfn, gensym("signal"), 0);
+    CLASS_MAINSIGNALIN(max_class, t_max, x_f);
     class_addmethod(max_class, (t_method)max_dsp, gensym("dsp"), 0);
     class_sethelpsymbol(max_class, gensym("sigbinops"));
     scalarmax_class = class_new(gensym("max~"), 0, 0,
     	sizeof(t_scalarmax), 0, 0);
-    class_addmethod(scalarmax_class, nullfn, gensym("signal"), 0);
+    CLASS_MAINSIGNALIN(scalarmax_class, t_scalarmax, x_f);
     class_addmethod(scalarmax_class, (t_method)scalarmax_dsp, gensym("dsp"),
     	0);
     class_sethelpsymbol(scalarmax_class, gensym("sigbinops"));
@@ -673,12 +693,14 @@ static t_class *min_class, *scalarmin_class;
 typedef struct _min
 {
     t_object x_obj;
+    float x_f;
 } t_min;
 
 typedef struct _scalarmin
 {
     t_object x_obj;
-    t_float x_f;
+    t_float x_g;
+    float x_f;
 } t_scalarmin;
 
 static void *min_new(t_symbol *s, int argc, t_atom *argv)
@@ -687,9 +709,10 @@ static void *min_new(t_symbol *s, int argc, t_atom *argv)
     if (argc) 
     {
 	t_scalarmin *x = (t_scalarmin *)pd_new(scalarmin_class);
-	floatinlet_new(&x->x_obj, &x->x_f);
-	x->x_f = atom_getfloatarg(0, argc, argv);
+	floatinlet_new(&x->x_obj, &x->x_g);
+	x->x_g = atom_getfloatarg(0, argc, argv);
 	outlet_new(&x->x_obj, &s_signal);
+    	x->x_f = 0;
 	return (x);
     }
     else
@@ -697,6 +720,7 @@ static void *min_new(t_symbol *s, int argc, t_atom *argv)
 	t_min *x = (t_min *)pd_new(min_class);
 	inlet_new(&x->x_obj, &x->x_obj.ob_pd, &s_signal, &s_signal);
 	outlet_new(&x->x_obj, &s_signal);
+    	x->x_f = 0;
 	return (x);
     }
 }
@@ -783,10 +807,10 @@ static void min_dsp(t_min *x, t_signal **sp)
 static void scalarmin_dsp(t_scalarmin *x, t_signal **sp)
 {
     if (sp[0]->s_n&7)
-    	dsp_add(scalarmin_perform, 4, sp[0]->s_vec, &x->x_f,
+    	dsp_add(scalarmin_perform, 4, sp[0]->s_vec, &x->x_g,
 	    sp[1]->s_vec, sp[0]->s_n);
     else	
-    	dsp_add(scalarmin_perf8, 4, sp[0]->s_vec, &x->x_f,
+    	dsp_add(scalarmin_perf8, 4, sp[0]->s_vec, &x->x_g,
 	    sp[1]->s_vec, sp[0]->s_n);
 }
 
@@ -794,12 +818,12 @@ static void min_setup(void)
 {
     min_class = class_new(gensym("min~"), (t_newmethod)min_new, 0,
     	sizeof(t_min), 0, A_GIMME, 0);
-    class_addmethod(min_class, nullfn, gensym("signal"), 0);
+    CLASS_MAINSIGNALIN(min_class, t_min, x_f);
     class_addmethod(min_class, (t_method)min_dsp, gensym("dsp"), 0);
     class_sethelpsymbol(min_class, gensym("sigbinops"));
     scalarmin_class = class_new(gensym("min~"), 0, 0,
     	sizeof(t_scalarmin), 0, 0);
-    class_addmethod(scalarmin_class, nullfn, gensym("signal"), 0);
+    CLASS_MAINSIGNALIN(scalarmin_class, t_scalarmin, x_f);
     class_addmethod(scalarmin_class, (t_method)scalarmin_dsp, gensym("dsp"),
     	0);
     class_sethelpsymbol(scalarmin_class, gensym("sigbinops"));
