@@ -20,6 +20,7 @@ int obj_nsiginlets(t_object *x);
 int obj_siginletindex(t_object *x, int m);
 int obj_nsigoutlets(t_object *x);
 int obj_sigoutletindex(t_object *x, int m);
+t_sample *obj_findsignalscalar(t_object *x, int m);
 static int ugen_loud;
 EXTERN_STRUCT _vinlet;
 EXTERN_STRUCT _voutlet;
@@ -155,7 +156,7 @@ static void *switch_new(t_floatarg fvecsize, t_floatarg foverlap)
     return (x);
 }
 
-static void block_float(t_block *x, t_floatarg *f)
+static void block_float(t_block *x, t_floatarg f)
 {
     if (x->x_switched)
     	x->x_switchon = (f != 0);
@@ -271,7 +272,7 @@ static t_signal *signal_freeborrowed;
     /* list of all signals allocated (not including "borrowed" ones) */
 static t_signal *signal_usedlist;
 
-    /* call this whien DSP is stopped to free all the signals */
+    /* call this when DSP is stopped to free all the signals */
 void signal_cleanup(void)
 {
     t_signal **svec, *sig, *sig2;
@@ -291,14 +292,35 @@ void signal_cleanup(void)
     /* mark the signal "reusable." */
 void signal_makereusable(t_signal *sig)
 {
+    int logn = ilog2(sig->s_n);
+#if 1
+    t_signal *s5;
+    for (s5 = signal_freeborrowed; s5; s5 = s5->s_nextfree)
+    {
+    	if (s5 == sig)
+	{
+	    bug("signal_free 3");
+	    return;
+	}
+    }
+    for (s5 = signal_freelist[logn]; s5; s5 = s5->s_nextfree)
+    {
+    	if (s5 == sig)
+	{
+	    bug("signal_free 4");
+	    return;
+	}
+    }
+#endif
     if (ugen_loud) post("free %x: %d", sig, sig->s_isborrowed);
     if (sig->s_isborrowed)
     {
     	    /* if the signal is borrowed, decrement the borowee's reference
 	    	count, possibly marking it reusable too */
     	t_signal *s2 = sig->s_borrowedfrom;
-	if (!s2)
+	if ((s2 == sig) || !s2)
 	    bug("signal_free");
+    	s2->s_refcount--;
 	if (!s2->s_refcount)
 	    signal_makereusable(s2);
 	sig->s_nextfree = signal_freeborrowed;
@@ -308,7 +330,6 @@ void signal_makereusable(t_signal *sig)
     {
     	    /* if it's a real signal, put it on the free list so we can
 	    reuse it. */
-    	int logn = ilog2(sig->s_n);
 	if (signal_freelist[logn] == sig) bug("signal_free 2");
     	sig->s_nextfree = signal_freelist[logn];
     	signal_freelist[logn] = sig;
@@ -614,15 +635,19 @@ static void ugen_doit(t_dspcontext *dc, t_ugenbox *u)
     
     if (ugen_loud) post("doit %s %d %d", class_getname(class), nofreesigs,
     	nonewsigs);
-    for (uin = u->u_in, i = u->u_nin; i--; uin++)
+    for (i = 0, uin = u->u_in; i < u->u_nin; i++, uin++)
     {
     	if (!uin->i_nconnect)
 	{
+    	    t_sample *scalar;
 	    s3 = signal_new(dc->dc_vecsize, dc->dc_srate);
     	    /* post("%s: unconnected signal inlet set to zero",
     	    	class_getname(u->u_obj->ob_pd)); */
-    	    dsp_add_zero(s3->s_vec, s3->s_n);
-    	    uin->i_signal = s3;
+	    if (scalar = obj_findsignalscalar(u->u_obj, i))
+	    	dsp_add_scalarcopy(scalar, s3->s_vec, s3->s_n);
+    	    else
+    	    	dsp_add_zero(s3->s_vec, s3->s_n);
+	    uin->i_signal = s3;
     	    s3->s_refcount = 1;
 	}
     }
@@ -635,8 +660,12 @@ static void ugen_doit(t_dspcontext *dc, t_ugenbox *u)
 	newrefcount = --(*sig)->s_refcount;
 	    /* if the reference count went to zero, we free the signal now,
 	    unless it's a subcanvas or outlet; these might keep the
-	    signal around to send to objects connected to them. */
-    	if (!newrefcount && !nofreesigs)
+	    signal around to send to objects connected to them.  In this
+	    case we increment the reference count; the corresponding decrement
+	    is in sig_makereusable(). */
+    	if (nofreesigs)
+	    (*sig)->s_refcount++;
+	else if (!newrefcount)
 	    signal_makereusable(*sig);
     }
     for (sig = outsig, uout = u->u_out, i = u->u_nout; i--; sig++, uout++)
@@ -649,8 +678,10 @@ static void ugen_doit(t_dspcontext *dc, t_ugenbox *u)
 	    since we've already freed the inputs the objects might get called
 	    "in place." */
     	if (nonewsigs)
+	{
     	    *sig = uout->o_signal =
 	    	signal_new(0, dc->dc_srate);
+	}
     	else
     	    *sig = uout->o_signal = signal_new(dc->dc_vecsize, dc->dc_srate);
 	(*sig)->s_refcount = uout->o_nconnect;
@@ -840,8 +871,9 @@ void ugen_done_graph(t_dspcontext *dc)
 	    for (i = 0, sigp = iosigs + dc->dc_ninlets; i < noutlets;
 	    	i++, sigp++)
 	    {
-	    	    signal_setborrowed(*sigp,
-		    	signal_new(parent_vecsize, parent_srate));
+    	    	signal_setborrowed(*sigp,
+    	    	    signal_new(parent_vecsize, parent_srate));
+		(*sigp)->s_refcount++;
 		if (ugen_loud) post("set %x->%x", *sigp, (*sigp)->s_borrowedfrom);
 	    }
 	} 
