@@ -4,61 +4,59 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <math.h>
 #include <stdio.h>  	/* for read/write to files */
 #include "m_pd.h"
 #include "g_canvas.h"
 
-/* ---------- pure arrays (cf. "garrays," graphical arrays, below) -------- */
+/* see also the "plot" object in g_scalar.c which deals with graphing
+arrays which are fields in scalars.  Someday we should unify the
+two, but how? */
+
+/* ---------- "pure" arrays whic have elements of scalars. ---- */
+/* ----------- cf. "garrays," graphical arrays, below. -------- */
 
 t_array *array_new(t_symbol *templatesym, t_gpointer *parent)
 {
     t_array *x = (t_array *)getbytes(sizeof (*x));
-    t_canvas *template =
+    t_canvas *templatecanvas =
     	(t_canvas *)pd_findbyclass(templatesym, canvas_class);
-    t_datatype *datatypes;
-    int elemsize, nitems;
+    t_template *template;
     t_gpointer *gp;
-    if (!template)
+    if (!templatecanvas)
     {
     	bug("%s: array: no such template", templatesym->s_name);
     	    /* LATER figure out how to recover from this? */
     	return (0);
     }
-    canvas_setusedastemplate(template);
-    elemsize = canvas_template_size(template);
-    datatypes = canvas_getdatatypes(template, &nitems);
+    canvas_setusedastemplate(templatecanvas);
+    template = canvas_gettemplate(templatecanvas);
     x->a_n = 1;
-    x->a_elemsize = elemsize;
-    x->a_vec = (t_word *)getbytes(elemsize);
+    x->a_elemsize = sizeof(t_word) * template->t_n;
+    x->a_vec = (t_word *)getbytes(x->a_elemsize);
+    	/* note here we blithely copy a gpointer instead of "setting" a
+	new one; this gpointer isn't accounted for and needn't be since
+	we'll be deleted before the thing pointed to gets deleted anyway;
+	see array_free. */
     x->a_gp = *parent;
     x->a_stub = gstub_new(0, x);
-    word_restore(x->a_vec, datatypes, nitems, 0, 0, parent);
-    freebytes(datatypes, nitems * sizeof(*datatypes));
+    word_init(x->a_vec, template, parent);
     return (x);
 }
 
-void array_resize(t_array *x, t_symbol *templatesym, int n)
+void array_resize(t_array *x, t_template *template, int n)
 {
-    int elemsize = x->a_elemsize;
-    int oldn = x->a_n;
-    t_canvas *template =
-    	(t_canvas *)pd_findbyclass(templatesym, canvas_class);
-    t_datatype *datatypes;
-    int nitems;
+    int elemsize, oldn;
     t_gpointer *gp;
 
-    if (!template)
-    {
-    	bug("%s: array: no such template", templatesym->s_name);
-    	return;
-    }
-    if (elemsize != canvas_template_size(template)) bug("array_resize");
-
-    datatypes = canvas_getdatatypes(template, &nitems);
-
-
+    if (n < 1)
+    	n = 1;
+    oldn = x->a_n;
+    elemsize = sizeof(t_word) * template->t_n;
+    
     x->a_vec = (t_word *)resizebytes(x->a_vec, oldn * elemsize,
     	n * elemsize);
+    x->a_n = n;
     if (n > oldn)
     {
     	char *cp = ((char *)x->a_vec) + elemsize * oldn;
@@ -66,14 +64,15 @@ void array_resize(t_array *x, t_symbol *templatesym, int n)
     	for (; i--; cp += elemsize)
     	{
     	    t_word *wp = (t_word *)cp;
-    	    word_restore(wp, datatypes, nitems, 0, 0, &x->a_gp);
+    	    word_init(wp, template, &x->a_gp);
     	}
     }
 }
 
 void array_free(t_array *x)
-{  
-    gpointer_unset(&x->a_gp);
+{
+	/* we don't unset our gpointer here since it was never "set." */
+    /* gpointer_unset(&x->a_gp); */
     gstub_cutoff(x->a_stub);
     freebytes(x->a_vec, x->a_elemsize * x->a_n);
     freebytes(x, sizeof *x);
@@ -97,41 +96,54 @@ struct _garray
     char x_usedindsp;	    /* true if some DSP routine is using this */
 };
 
-void graph_array(t_glist *gl, t_symbol *s, t_symbol *template, t_floatarg f)
+void graph_array(t_glist *gl, t_symbol *s, t_symbol *templatesym, t_floatarg f)
 {
-    int n = f;
+    int n = f, i;
     static int gcount = 0;
-    int zz;
+    int zz, nwords;
     t_garray *x;
-    t_canvas *c;
+    t_template *template;
     char *str;
     if (s == &s_)
     {
     	char buf[40];
     	sprintf(buf, "array%d", ++gcount);
     	s = gensym(buf);    	
-    	template = &s_float;
+    	templatesym = &s_float;
     	n = 100;
     }
     else if (!strncmp((str = s->s_name), "array", 5)
     	&& (zz = atoi(str + 5)) > gcount) gcount = zz;
-    c = (t_canvas *)pd_findbyclass(template, canvas_class);
-    if (!c && template != &s_float)
+    template = canvas_gettemplatebyname(templatesym);
+    if (!template)
     {
-    	error("array: couldn't find template %s", template->s_name);
+    	error("array: couldn't find template %s", templatesym->s_name);
     	return;
     }
+    nwords =  template->t_n;
+    for (i = 0; i < nwords; i++)
+    {
+    	    /* we can't have array or list elements yet because what scalar
+	    can act as their "parent"??? */
+    	if (template->t_vec[i].ds_type == DT_ARRAY
+	    || template->t_vec[i].ds_type == DT_LIST)
+	{
+	    error("array: template %s can't have sublists or arrays",
+	    	templatesym->s_name);
+    	    return;
+    	}
+    }
     x = (t_garray *)pd_new(garray_class);
-    
+
     pd_bind(&x->x_gobj.g_pd, s);
     if (n <= 0) n = 100;  
     x->x_n = n;
-    x->x_elemsize = canvas_template_size(c);
+    x->x_elemsize = nwords * sizeof(t_word);
     x->x_vec = getbytes(x->x_n * x->x_elemsize);
     memset(x->x_vec, 0, x->x_n * x->x_elemsize);
     	/* LATER should check that malloc */
     x->x_name = s;
-    x->x_template = template;
+    x->x_template = templatesym;
     x->x_firstx = 0;
     x->x_xinc = 1;	    	/* LATER make methods to set this... */
     glist_add(gl, &x->x_gobj);
@@ -190,20 +202,21 @@ static void garray_vis(t_gobj *z, t_glist *glist, int vis)
     t_garray *x = (t_garray *)z;
     if (vis)
     {
-    	int i, elemsize, xonset, yonset;
-    	t_field *fx, *fy;
-    	t_canvas *c = (t_canvas *)pd_findbyclass(x->x_template, canvas_class);
-    	elemsize = canvas_template_size(c);
-    	if (!(fy = canvas_find_field(c, gensym("y"), &yonset))
-    	    || field_type(fy) != DT_FLOAT)
+    	int i, xonset, yonset, type;
+	t_symbol *arraytype;
+	t_template *template = canvas_gettemplatebyname(x->x_template);
+	if (!template)
+	    return;
+    	if (!template_find_field(template, gensym("y"), &yonset, &type,
+	    &arraytype) || type != DT_FLOAT)
     	{
     	    error("%s: needs floating-point 'y' field", x->x_template->s_name);
     	    sys_vgui(".x%x.c create text 50 50 -text foo\
     	    	-tags .x%x.a%x\n",
     	    	glist_getcanvas(glist), glist_getcanvas(glist), x);
     	}
-    	else if (!(fx = canvas_find_field(c, gensym("x"), &xonset))
-    	    || field_type(fx) != DT_FLOAT)
+    	else if (!template_find_field(template, gensym("x"), &xonset, &type,
+	    &arraytype) || type != DT_FLOAT)
     	{
     	    float firsty, xcum = x->x_firstx;
     	    int lastpixel = -1, ndrawn = 0;
@@ -212,7 +225,8 @@ static void garray_vis(t_gobj *z, t_glist *glist, int vis)
     	    sys_vgui(".x%x.c create line \\\n", glist_getcanvas(glist));
     	    for (i = 0; i < x->x_n; i++)
     	    {
-    	    	yval = *(float *)((x->x_vec + elemsize * i) + yonset);
+    	    	yval = *(float *)(x->x_vec +
+		    template->t_n * i * sizeof (t_word) + yonset);
     	    	xpix = glist_xtopixels(glist, xcum);
     	    	ixpix = xpix + 0.5;
     	    	if (ixpix != lastpixel)
@@ -251,6 +265,13 @@ static void garray_vis(t_gobj *z, t_glist *glist, int vis)
     }
 }
 
+static int garray_click(t_gobj *z, struct _glist *glist,
+    int xpix, int ypix, int shift, int alt, int dbl, int doit)
+{
+    	/* LATER fill this in */
+    return (0);
+}
+
 static void garray_save(t_gobj *z, t_binbuf *b)
 {
     t_garray *x = (t_garray *)z;
@@ -266,7 +287,9 @@ t_widgetbehavior garray_widgetbehavior =
     garray_activate,
     garray_delete,
     garray_vis,
-    garray_save
+    garray_click,
+    garray_save,
+    0
 };
 
 /* ----------------------- public functions -------------------- */
@@ -285,9 +308,15 @@ void garray_redraw(t_garray *x)
     }
 }
 
-t_canvas *garray_template(t_garray *x)	/* get the template of an array */
+   /* This functiopn gets the template of an array; if we can't figure
+   out what template an array's elements belong to we're in grave trouble
+   when it's time to free or resize it.  */
+t_template *garray_template(t_garray *x)
 {
-    return ((t_canvas *)pd_findbyclass(x->x_template, canvas_class));
+    t_template *template = canvas_gettemplatebyname(x->x_template);
+    if (!template)
+    	bug("garray_template");
+    return (template);
 }
 
 int garray_npoints(t_garray *x)	/* get the length */
@@ -305,14 +334,13 @@ char *garray_vec(t_garray *x) /* get the contents */
 
 int garray_getfloatarray(t_garray *x, int *size, t_float **vec)
 {
-    t_canvas *c = garray_template(x);
-    int elemsize = canvas_template_size(c);
-    t_field *fy;
-    int yonset;
-    if (!(fy = canvas_find_field(c, gensym("y"), &yonset))
-    	|| field_type(fy) != DT_FLOAT)
+    t_template *template = garray_template(x);
+    int yonset, type;
+    t_symbol *arraytype;
+    if (!template_find_field(template, gensym("y"), &yonset,
+    	&type, &arraytype) || type != DT_FLOAT)
     	    error("%s: needs floating-point 'y' field", x->x_template->s_name);
-    else if (elemsize != sizeof(float))
+    else if (template->t_n != 1)
     	error("%s: has more than one field", x->x_template->s_name);
     else
     {
@@ -326,12 +354,11 @@ int garray_getfloatarray(t_garray *x, int *size, t_float **vec)
     /* get any floating-point field of any element of an array */
 float garray_get(t_garray *x, t_symbol *s, t_int indx)
 {
-    int elemsize, yonset;
-    t_field *fy;
-    t_canvas *c = (t_canvas *)pd_findbyclass(x->x_template, canvas_class);
-    elemsize = canvas_template_size(c);
-    if (!(fy = canvas_find_field(c, s, &yonset))
-    	|| field_type(fy) != DT_FLOAT)
+    t_template *template = garray_template(x);
+    int yonset, type;
+    t_symbol *arraytype;
+    if (!template_find_field(template, gensym("y"), &yonset,
+    	&type, &arraytype) || type != DT_FLOAT)
     {
     	error("%s: needs floating-point '%s' field", x->x_template->s_name,
     	    s->s_name);
@@ -339,21 +366,141 @@ float garray_get(t_garray *x, t_symbol *s, t_int indx)
     }
     if (indx < 0) indx = 0;
     else if (indx >= x->x_n) indx = x->x_n - 1;
-    return (*(float *)((x->x_vec + elemsize * indx) + yonset));
+    return (*(float *)((x->x_vec + sizeof(t_word) * indx) + yonset));
 }
 
 /*------------------- Pd messages ------------------------ */
 static void garray_const(t_garray *x, t_floatarg g)
 {
-    int i, elemsize, yonset;
-    t_field *fy;
-    t_canvas *c = (t_canvas *)pd_findbyclass(x->x_template, canvas_class);
-    elemsize = canvas_template_size(c);
-    if (!(fy = canvas_find_field(c, gensym("y"), &yonset))
-    	|| field_type(fy) != DT_FLOAT)
+    t_template *template = garray_template(x);
+    int yonset, type, i;
+    t_symbol *arraytype;
+    if (!template_find_field(template, gensym("y"), &yonset,
+    	&type, &arraytype) || type != DT_FLOAT)
     	    error("%s: needs floating-point 'y' field", x->x_template->s_name);
     else for (i = 0; i < x->x_n; i++)
-    	*(float *)((x->x_vec + elemsize * i) + yonset) = g;
+    	*(float *)((x->x_vec + sizeof(t_word) * i) + yonset) = g;
+    garray_redraw(x);
+}
+
+    /* sum of Fourier components; called from routines below */
+static void garray_dofo(t_garray *x, int npoints, float dcval,
+    int nsin, t_float *vsin, int sineflag)
+{
+    t_template *template = garray_template(x);
+    int yonset, type, i, j;
+    t_symbol *arraytype;
+    double phase, phaseincr, fj;
+    if (npoints == 0)
+    	npoints = 512;	/* dunno what a good default would be... */
+    if (npoints != (1 << ilog2(npoints)))
+    	post("%s: rounnding to %d points", x->x_template->s_name,
+	    (npoints = (1<<ilog2(npoints))));
+    garray_resize(x, npoints + 3);
+    phaseincr = 2. * 3.14159 / npoints;
+    if (!template_find_field(template, gensym("y"), &yonset,
+    	&type, &arraytype) || type != DT_FLOAT)
+    {
+    	error("%s: needs floating-point 'y' field", x->x_template->s_name);
+	return;
+    }
+    for (i = 0, phase = -phaseincr; i < x->x_n; i++, phase += phaseincr )
+    {
+    	double sum = dcval;
+	if (sineflag)
+	    for (j = 0, fj = phase; j < nsin; j++, fj += phase)
+	    	sum += vsin[j] * sin(fj);
+	else
+	    for (j = 0, fj = 0; j < nsin; j++, fj += phase)
+	    	sum += vsin[j] * cos(fj);
+    	*(float *)((x->x_vec + sizeof(t_word) * i) + yonset) = sum;
+    }
+    garray_redraw(x);
+}
+
+static void garray_sinesum(t_garray *x, t_symbol *s, int argc, t_atom *argv)
+{
+    t_template *template = garray_template(x);
+    
+    t_float *svec = (t_float *)t_getbytes(sizeof(t_float) * argc);
+    int npoints, i;
+    if (argc < 2)
+    {
+    	error("sinesum: %s: need number of points and partial strengths",
+	    x->x_template->s_name);
+	return;
+    }
+
+    npoints = atom_getfloatarg(0, argc, argv);
+    argv++, argc--;
+    
+    svec = (t_float *)t_getbytes(sizeof(t_float) * argc);
+    if (!svec) return;
+    
+    for (i = 0; i < argc; i++)
+    	svec[i] = atom_getfloatarg(i, argc, argv);
+    garray_dofo(x, npoints, 0, argc, svec, 1);
+    t_freebytes(svec, sizeof(t_float) * argc);
+}
+
+static void garray_cosinesum(t_garray *x, t_symbol *s, int argc, t_atom *argv)
+{
+    t_template *template = garray_template(x);
+    
+    t_float *svec = (t_float *)t_getbytes(sizeof(t_float) * argc);
+    int npoints, i;
+    if (argc < 2)
+    {
+    	error("sinesum: %s: need number of points and partial strengths",
+	    x->x_template->s_name);
+	return;
+    }
+
+    npoints = atom_getfloatarg(0, argc, argv);
+    argv++, argc--;
+    
+    svec = (t_float *)t_getbytes(sizeof(t_float) * argc);
+    if (!svec) return;
+
+    for (i = 0; i < argc; i++)
+    	svec[i] = atom_getfloatarg(i, argc, argv);
+    garray_dofo(x, npoints, 0, argc, svec, 0);
+    t_freebytes(svec, sizeof(t_float) * argc);
+}
+
+static void garray_normalize(t_garray *x, t_float f)
+{
+    t_template *template = garray_template(x);
+    int yonset, type, npoints, i;
+    double maxv, renormer;
+    t_symbol *arraytype;
+    
+    if (f <= 0)
+    	f = 1;
+
+    if (!template_find_field(template, gensym("y"), &yonset,
+    	&type, &arraytype) || type != DT_FLOAT)
+    {
+    	error("%s: needs floating-point 'y' field", x->x_template->s_name);
+	return;
+    }
+    for (i = 0, maxv = 0; i < x->x_n; i++)
+    {
+    	double v = *(float *)((x->x_vec + sizeof(t_word) * i) + yonset);
+	if (v > maxv)
+	    maxv = v;
+	if (-v > maxv)
+	    maxv = -v;
+    }
+    if (maxv >= 0)
+    {
+	renormer = f / maxv;
+	for (i = 0; i < x->x_n; i++)
+	{
+    	    *(float *)((x->x_vec + sizeof(t_word) * i) + yonset)
+	    	*= renormer;
+	}
+    }
     garray_redraw(x);
 }
 
@@ -361,12 +508,11 @@ static void garray_const(t_garray *x, t_floatarg g)
     the "y" slot of the array.  This generalizes Max's "table", sort of. */
 static void garray_list(t_garray *x, t_symbol *s, int argc, t_atom *argv)
 {
-    int i, elemsize, yonset;
-    t_field *fy;
-    t_canvas *c = (t_canvas *)pd_findbyclass(x->x_template, canvas_class);
-    elemsize = canvas_template_size(c);
-    if (!(fy = canvas_find_field(c, gensym("y"), &yonset))
-    	|| field_type(fy) != DT_FLOAT)
+    t_template *template = garray_template(x);
+    int yonset, type, i;
+    t_symbol *arraytype;
+    if (!template_find_field(template, gensym("y"), &yonset,
+    	&type, &arraytype) || type != DT_FLOAT)
     	    error("%s: needs floating-point 'y' field", x->x_template->s_name);
     else if (argc < 2) return;
     else
@@ -388,7 +534,7 @@ static void garray_list(t_garray *x, t_symbol *s, int argc, t_atom *argv)
     	    if (argc <= 0) return;
     	}
     	for (i = 0; i < argc; i++)
-    	    *(float *)((x->x_vec + elemsize * (i + firstindex)) + yonset) = 
+    	    *(float *)((x->x_vec + sizeof(t_word) * (i + firstindex)) + yonset) = 
     	    	atom_getfloat(argv + i);
     }
     garray_redraw(x);
@@ -433,14 +579,14 @@ static void garray_rename(t_garray *x, t_symbol *s)
 
 static void garray_read(t_garray *x, t_symbol *filename)
 {
-    int i, nelem = x->x_n, elemsize, yonset, filedesc;
-    t_field *fy;
+    int nelem = x->x_n, filedesc;
     FILE *fd;
-    t_canvas *c = (t_canvas *)pd_findbyclass(x->x_template, canvas_class);
     char buf[MAXPDSTRING], *bufptr;
-    elemsize = canvas_template_size(c);
-    if (!(fy = canvas_find_field(c, gensym("y"), &yonset))
-    	|| field_type(fy) != DT_FLOAT)
+    t_template *template = garray_template(x);
+    int yonset, type, i;
+    t_symbol *arraytype;
+    if (!template_find_field(template, gensym("y"), &yonset,
+    	&type, &arraytype) || type != DT_FLOAT)
     {
     	error("%s: needs floating-point 'y' field", x->x_template->s_name);
     	return;
@@ -455,7 +601,8 @@ static void garray_read(t_garray *x, t_symbol *filename)
     }
     for (i = 0; i < nelem; i++)
     {
-    	if (!fscanf(fd, "%f", (float *)((x->x_vec + elemsize * i) + yonset)))
+    	if (!fscanf(fd, "%f", (float *)((x->x_vec + sizeof(t_word) * i) +
+	    yonset)))
     	{
     	    post("%s: read %d elements into table of size %d",
     	    	filename->s_name, i, nelem);
@@ -463,25 +610,21 @@ static void garray_read(t_garray *x, t_symbol *filename)
     	}
     }
     while (i < nelem)
-    	*(float *)((x->x_vec + elemsize * i) + yonset) = 0, i++;
+    	*(float *)((x->x_vec + sizeof(t_word) * i) + yonset) = 0, i++;
     fclose(fd);
     garray_redraw(x);
 }
 
-static int garray_ambigendian(void)
+    /* this should be renamed and moved... */
+int garray_ambigendian(void)
 {
     unsigned short s = 1;
     unsigned char c = *(char *)(&s);
     return (c==0);
 }
 
-#ifdef NT
 #define BINREADMODE "rb"
 #define BINWRITEMODE "wb"
-#else
-#define BINREADMODE "r"
-#define BINWRITEMODE "w"
-#endif
 
 static void garray_read16(t_garray *x, t_symbol *filename,
     t_symbol *endian, t_floatarg fskip)
@@ -549,14 +692,13 @@ static void garray_read16(t_garray *x, t_symbol *filename,
 
 static void garray_write(t_garray *x, t_symbol *filename)
 {
-    int i, elemsize, yonset;
-    t_field *fy;
     FILE *fd;
-    t_canvas *c = (t_canvas *)pd_findbyclass(x->x_template, canvas_class);
     char buf[MAXPDSTRING];
-    elemsize = canvas_template_size(c);
-    if (!(fy = canvas_find_field(c, gensym("y"), &yonset))
-    	|| field_type(fy) != DT_FLOAT)
+    t_template *template = garray_template(x);
+    int yonset, type, i;
+    t_symbol *arraytype;
+    if (!template_find_field(template, gensym("y"), &yonset,
+    	&type, &arraytype) || type != DT_FLOAT)
     {
     	error("%s: needs floating-point 'y' field", x->x_template->s_name);
     	return;
@@ -572,7 +714,7 @@ static void garray_write(t_garray *x, t_symbol *filename)
     for (i = 0; i < x->x_n; i++)
     {
     	if (fprintf(fd, "%g\n",
-    	    *(float *)((x->x_vec + elemsize * i) + yonset)) < 1)
+    	    *(float *)((x->x_vec + sizeof(t_word) * i) + yonset)) < 1)
     	{
     	    post("%s: write error", filename->s_name);
     	    break;
@@ -597,14 +739,13 @@ static unsigned char waveheader[] = {
 0x00, 0x00, 0x00, 0x00,
 };
 
-
     /* wave format only so far */
 static void garray_write16(t_garray *x, t_symbol *filename, t_symbol *format)
 {
-    int i, elemsize, yonset;
-    t_field *fy;
+    t_template *template = garray_template(x);
+    int yonset, type, i;
+    t_symbol *arraytype;
     FILE *fd;
-    t_canvas *c = (t_canvas *)pd_findbyclass(x->x_template, canvas_class);
     int aiff = (format == gensym("aiff"));
     char filenamebuf[MAXPDSTRING], buf2[MAXPDSTRING];
     int swap = garray_ambigendian();	/* wave is only little endian */
@@ -622,9 +763,8 @@ static void garray_write16(t_garray *x, t_symbol *filename, t_symbol *format)
     	if (strcmp(filenamebuf + strlen(filenamebuf)-4, ".wav"))
     	    strcat(filenamebuf, ".wav");
     }
-    elemsize = canvas_template_size(c);
-    if (!(fy = canvas_find_field(c, gensym("y"), &yonset))
-    	|| field_type(fy) != DT_FLOAT)
+    if (!template_find_field(template, gensym("y"), &yonset,
+    	&type, &arraytype) || type != DT_FLOAT)
     {
     	error("%s: needs floating-point 'y' field", x->x_template->s_name);
     	return;
@@ -660,7 +800,7 @@ static void garray_write16(t_garray *x, t_symbol *filename, t_symbol *format)
     }
     for (i = 0; i < x->x_n; i++)
     {
-    	float f = 32767. * *(float *)((x->x_vec + elemsize * i) + yonset);
+    	float f = 32767. * *(float *)((x->x_vec + sizeof(t_word) * i) + yonset);
     	short sh;
     	if (f < -32768) f = -32768;
     	else if (f > 32767) f = 32767;
@@ -680,18 +820,25 @@ closeit:
     fclose(fd);
 }
 
-static void garray_resize(t_garray *x, t_floatarg f)
+void garray_resize(t_garray *x, t_floatarg f)
 {
     int was = x->x_n, elemsize;
     t_glist *gl;
     t_canvas *c;
     int dspwas;
     int n = f;
+    char *nvec;
+    
     if (n < 0) n = 0;
     c = (t_canvas *)pd_findbyclass(x->x_template, canvas_class);
-    elemsize = canvas_template_size(c);
-
-    x->x_vec = t_resizebytes(x->x_vec, was * elemsize, n * elemsize);
+    elemsize = canvas_gettemplatebyname(x->x_template)->t_n * sizeof(t_word);
+    nvec = t_resizebytes(x->x_vec, was * elemsize, n * elemsize);
+    if (!nvec)
+    {
+    	pd_error(x, "array resize failed: out of memory");
+	return;
+    }
+    x->x_vec = nvec;
     	/* LATER should check t_resizebytes result */
     if (n > was)
     	memset(x->x_vec + was*elemsize,
@@ -704,7 +851,7 @@ static void garray_resize(t_garray *x, t_floatarg f)
     if (gl->gl_list == &x->x_gobj && !x->x_gobj.g_next)
     {
     	vmess(&gl->gl_pd, gensym("bounds"), "ffff",
-    	    0., gl->gl_y1, (double)n, gl->gl_y2);
+    	    0., gl->gl_y1, (double)(n > 1 ? n-1 : 1), gl->gl_y2);
     }
     else garray_redraw(x);
     if (x->x_usedindsp) canvas_update_dsp();
@@ -748,5 +895,11 @@ void g_array_setup(void)
     	A_FLOAT, A_NULL);
     class_addmethod(garray_class, (t_method)garray_print, gensym("print"),
     	A_NULL);
+    class_addmethod(garray_class, (t_method)garray_sinesum, gensym("sinesum"),
+    	A_GIMME, 0);
+    class_addmethod(garray_class, (t_method)garray_cosinesum,
+    	gensym("cosinesum"), A_GIMME, 0);
+    class_addmethod(garray_class, (t_method)garray_normalize,
+    	gensym("normalize"), A_DEFFLOAT, 0);
 }
 
