@@ -27,8 +27,6 @@ void glist_add(t_glist *x, t_gobj *y)
     if (canvas_isvisible(glist_getcanvas(x))) gobj_vis(y, x, 1);
     if (class_isdrawcommand(y->g_pd)) 
     	canvas_redrawallfortemplate(glist_getcanvas(x));
-    if (y->g_pd == field_class)
-    	canvas_zapallfortemplate(glist_getcanvas(x));
 }
 
     /* delete an object from a glist and free it */
@@ -37,7 +35,6 @@ void glist_delete(t_glist *x, t_gobj *y)
     t_gobj *g;
     t_gotfn chkdsp = zgetfn(&y->g_pd, gensym("dsp"));
     int drawcommand = class_isdrawcommand(y->g_pd);
-    int field = (y->g_pd == field_class);
     	/*  LATER decide whether all visible glists must have an editor? */
     if (canvas_isvisible(glist_getcanvas(x)) && x->gl_editor)
     {
@@ -55,11 +52,20 @@ void glist_delete(t_glist *x, t_gobj *y)
     }
     pd_free(&y->g_pd);
     if (chkdsp) canvas_update_dsp();
-    if (field) canvas_zapallfortemplate(glist_getcanvas(x));
     if (drawcommand) canvas_redrawallfortemplate(glist_getcanvas(x));
     x->gl_valid = ++glist_valid;
 }
 
+    /* remove every object from a glist.  Experimental. */
+void glist_clear(t_glist *x)
+{
+    t_gobj *y, *y2;
+    int dspstate = canvas_suspend_dsp();
+    while (y = x->gl_list)
+    	glist_delete(x, y);
+    canvas_resume_dsp(dspstate);
+}
+
 void glist_retext(t_glist *glist, t_text *y)
 {
     t_canvas *c = glist_getcanvas(glist);
@@ -70,10 +76,16 @@ void glist_retext(t_glist *glist, t_text *y)
     }
 }
 
-void glist_grab(t_glist *x, t_gobj *y, int xpos, int ypos)
+void glist_grab(t_glist *x, t_gobj *y, t_glistmotionfn motionfn,
+    t_glistkeyfn keyfn, int xpos, int ypos)
 {
+    t_glist *x2;
+    while (x2 = x->gl_owner)
+    	x = x2;
     x->gl_editor->e_onmotion = MA_PASSOUT;
     x->gl_editor->e_grab = y;
+    x->gl_editor->e_motionfn = motionfn;
+    x->gl_editor->e_keyfn = keyfn;
     x->gl_editor->e_xwas = xpos;
     x->gl_editor->e_ywas = ypos;
 }
@@ -95,6 +107,96 @@ int glist_getfont(t_glist *x)
     return (canvas_getfont(glist_getcanvas(x)));
 }
 
+static float gobj_getxforsort(t_gobj *g)
+{
+    if (pd_class(&g->g_pd) == scalar_class)
+    {
+    	float x1, y1;
+    	scalar_getbasexy((t_scalar *)g, &x1, &y1);
+	return(x1);
+    }
+    else return (0);
+}
+
+static t_gobj* glist_merge(t_glist *x, t_gobj *g1, t_gobj *g2)
+{
+    t_gobj *g = 0, *g9 = 0;
+    float f1 = 0, f2 = 0;
+    if (g1)
+    	f1 = gobj_getxforsort(g1);
+    if (g2)
+    	f2 = gobj_getxforsort(g2);
+    while (1)
+    {
+    	if (g1)
+	{
+	    if (g2)
+	    {
+	    	if (f1 <= f2)
+		    goto put1;
+		else goto put2;
+	    }
+	    else goto put1;	
+	}
+	else if (g2)
+	    goto put2;
+	else break;
+    put1:
+    	if (g9)
+	    g9->g_next = g1, g9 = g1;
+	else g9 = g = g1;
+	if (g1 = g1->g_next)
+	    f1 = gobj_getxforsort(g1);
+    	g9->g_next = 0;
+	continue;
+    put2:
+    	if (g9)
+	    g9->g_next = g2, g9 = g2;
+	else g9 = g = g2;
+	if (g2 = g2->g_next)
+	    f2 = gobj_getxforsort(g2);
+    	g9->g_next = 0;
+	continue;
+    }
+    return (g);
+}
+
+static t_gobj *glist_dosort(t_glist *x,
+    t_gobj *g, int nitems)
+{
+    if (nitems < 2)
+    	return (g);
+    else
+    {
+    	int n1 = nitems/2, n2 = nitems - n1, i;
+	t_gobj *g2, *g3;
+	for (g2 = g, i = n1-1; i--; g2 = g2->g_next)
+	    ;
+	g3 = g2->g_next;
+	g2->g_next = 0;
+	g = glist_dosort(x, g, n1);
+	g3 = glist_dosort(x, g3, n2);
+	return (glist_merge(x, g, g3));
+    }
+}
+
+void glist_sort(t_glist *x)
+{
+    int nitems = 0, foo = 0;
+    float lastx = -1e37;
+    t_gobj *g;
+    for (g = x->gl_list; g; g = g->g_next)
+    {
+    	float x1 = gobj_getxforsort(g);
+	if (x1 < lastx)
+	    foo = 1;
+    	lastx = x1;
+	nitems++;
+    }
+    if (foo)
+    	x->gl_list = glist_dosort(x, x->gl_list, nitems);
+}
+
 void glist_init(t_glist *x)
 {
     x->gl_list = 0;
@@ -113,9 +215,24 @@ void glist_init(t_glist *x)
 
 void glist_cleanup(t_glist *x)
 {
+    freebytes(x->gl_xlabel, x->gl_nxlabels * sizeof(*(x->gl_xlabel)));
+    freebytes(x->gl_ylabel, x->gl_nylabels * sizeof(*(x->gl_ylabel)));
     gstub_cutoff(x->gl_stub);
 }
 
+void glist_free(t_glist *x)
+{
+    glist_cleanup(x);
+    freebytes(x, sizeof(*x));
+}
+
+t_glist *glist_new( void)
+{
+    t_glist *x = (t_glist *)getbytes(sizeof(*x));
+    glist_init(x);
+    return (x);
+}
+
 /* --------------------- functions peculiar to graphs -------------- */
 
 static void graph_redraw(t_glist *x);
@@ -388,7 +505,72 @@ static void graph_delete(t_gobj *z, t_glist *glist)
     	sys_vgui(".x%x.c delete graph%x\n", glist_getcanvas(glist), x);
 }
 
-void graph_save(t_gobj *z, t_binbuf *b)
+static float graph_lastxpix, graph_lastypix;
+
+static void graph_motion(void *z, t_floatarg dx, t_floatarg dy)
+{
+    t_glist *x = (t_glist *)z;
+    float newxpix = graph_lastxpix + dx, newypix = graph_lastypix + dy;
+    t_garray *a = (t_garray *)(x->gl_list);
+    int oldx = 0.5 + glist_pixelstox(x, graph_lastxpix);
+    int newx = 0.5 + glist_pixelstox(x, newxpix);
+    t_float *vec;
+    int nelem, i;
+    float oldy = glist_pixelstoy(x, graph_lastypix);
+    float newy = glist_pixelstoy(x, newypix);
+    graph_lastxpix = newxpix;
+    graph_lastypix = newypix;
+    	/* verify that the array is OK */
+    if (!a || pd_class((t_pd *)a) != garray_class)
+    	return;
+    if (!garray_getfloatarray(a, &nelem, &vec))
+    	return;
+    if (oldx < 0) oldx = 0;
+    if (oldx >= nelem)
+    	oldx = nelem - 1;
+    if (newx < 0) newx = 0;
+    if (newx >= nelem)
+    	newx = nelem - 1;
+    if (oldx < newx - 1)
+    {
+    	for (i = oldx + 1; i <= newx; i++)
+	    vec[i] = newy + (oldy - newy) *
+	    	((float)(newx - i))/(float)(newx - oldx);
+    }
+    else if (oldx > newx + 1)
+    {
+    	for (i = oldx - 1; i >= newx; i--)
+	    vec[i] = newy + (oldy - newy) *
+	    	((float)(newx - i))/(float)(newx - oldx);
+    }
+    else vec[newx] = newy;
+    garray_redraw(a);
+}
+
+static int graph_click(t_gobj *z, struct _glist *glist,
+    int xpix, int ypix, int shift, int alt, int dbl, int doit)
+{
+    t_glist *x = (t_glist *)z;
+    if (x->gl_px1 >= x->gl_px2 || x->gl_x2 - x->gl_x1 > x->gl_px2 - x->gl_px1)
+    {
+    	if (doit)
+    	    pd_error(x,
+	    	"sorry, graph drawing needs at least one pixel per element");
+    	return (0);
+    }
+    else
+    {
+    	if (doit)
+	{
+    	    graph_lastxpix = xpix;
+    	    graph_lastypix = ypix;
+    	    glist_grab(x->gl_owner, &x->gl_gobj, graph_motion, 0, xpix, ypix);
+    	}
+	return (1);
+    }
+}
+
+static void graph_save(t_gobj *z, t_binbuf *b)
 {
     t_gobj *y;
     t_glist *x = (t_glist *)z;
@@ -402,6 +584,16 @@ void graph_save(t_gobj *z, t_binbuf *b)
     binbuf_addv(b, "ss;", gensym("#X"), gensym("pop"));
 }
 
+static void graph_properties(t_gobj *z, t_glist *owner)
+{
+    t_glist *x = (t_glist *)z;
+    char graphbuf[200];
+    sprintf(graphbuf, "pdtk_graph_dialog %%s %g %g %g %g %g %g\n",
+	x->gl_x1, x->gl_y1, x->gl_x2, x->gl_y2,
+	    x->gl_px2 - x->gl_px1, x->gl_py1 - x->gl_py2);
+    gfxstub_new(&x->gl_pd, x, graphbuf);
+}
+
 extern void graph_vis(t_gobj *x, t_glist *glist, int flag);
 
 t_widgetbehavior graph_widgetbehavior =
@@ -412,7 +604,9 @@ t_widgetbehavior graph_widgetbehavior =
     graph_activate,
     graph_delete,
     graph_vis,
+    graph_click,
     graph_save,
+    graph_properties,
 };
 
 
@@ -442,10 +636,11 @@ void glist_glist(t_glist *g, t_symbol *s, int argc, t_atom *argv)
     	sym = gensym(buf);
     	menu = 1;
     }
-    else if (!strncmp((str = s->s_name), "graph", 5)
-    	&& (zz = atoi(str + 5)) > gcount) gcount = zz;
+    else if (!strncmp((str = sym->s_name), "graph", 5)
+    	&& (zz = atoi(str + 5)) > gcount)
+	    gcount = zz;
     if (x1 == x2 || y1 == y2) x1 = 0, x2 = 100, y1 = -1, y2 = 1;
-    if (px1 == px2 || py1 == py2) px1 = 200, py1 = 320, px2 = 600, py2 = 20;
+    if (px1 == px2 || py1 == py2) px1 = 100, py1 = 160, px2 = 300, py2 = 20;
     x->gl_name = sym;
     x->gl_x1 = x1;
     x->gl_x2 = x2;
@@ -471,28 +666,10 @@ t_glist *glist_findgraph(t_glist *x, float x1, float x2, float y1, float y2)
     for (z = x->gl_list; z; z = z->g_next)
     	if (pd_class(&z->g_pd) == graph_class) y = z;
     if (y) return ((t_glist *)y);
-    vmess(&x->gl_pd, gensym("graph"), "");
+    pd_vmess(&x->gl_pd, gensym("graph"), "");
     return (glist_findgraph(x, 0, 0, 0, 0));
 }
 
-void graph_free(t_glist *x) 	/* LATER compare me with canvas_free() */
-{
-    t_gobj *y;
-    while (y = x->gl_list) glist_delete(x, y);
-    pd_unbind(&x->gl_gobj.g_pd, x->gl_name);
-    glist_cleanup(x);
-}
-
-static void graph_click(t_glist *x,
-    t_floatarg xpos, t_floatarg ypos,
-    	t_floatarg shift, t_floatarg ctrl, t_floatarg alt)
-{
-  float xk,yk;
-  xk = glist_pixelstox(x,xpos);
-  yk = glist_pixelstoy(x,ypos);
-/*  post("graph clicked at <%f,%f> <%f,%f>",xpos,ypos,xk,yk); */
-}
-
 static void graph_dialog(t_glist *x, t_symbol *s, int argc, t_atom *argv)
 {
     t_float x1 = atom_getfloatarg(0, argc, argv);
@@ -512,6 +689,16 @@ static void graph_dialog(t_glist *x, t_symbol *s, int argc, t_atom *argv)
     }
 }
 
+void graph_free(t_glist *x) 	/* LATER compare me with canvas_free() */
+{
+    t_gobj *y;
+    while (y = x->gl_list) glist_delete(x, y);
+    pd_unbind(&x->gl_gobj.g_pd, x->gl_name);
+    gfxstub_deleteforkey(x);
+
+    glist_cleanup(x);
+}
+
 extern void glist_text(t_glist *gl, t_symbol *s, int argc, t_atom *argv);
 
 void g_graph_setup(void)
@@ -534,8 +721,14 @@ void g_graph_setup(void)
     	A_GIMME, A_NULL);
     class_addmethod(graph_class, (t_method)graph_array, gensym("array"),
     	A_SYMBOL, A_FLOAT, A_SYMBOL, A_NULL);
-    class_addmethod(graph_class, (t_method)graph_click,
-    	gensym("click"), A_FLOAT, A_FLOAT, A_FLOAT, 0);
     class_addmethod(graph_class, (t_method)graph_dialog,
     	gensym("dialog"), A_GIMME, 0);
+    class_addmethod(graph_class, (t_method)glist_clear, gensym("clear"),
+    	A_NULL);
+    class_addmethod(graph_class, (t_method)glist_write,
+    	gensym("write"), A_SYMBOL, A_DEFSYM, A_NULL);
+    class_addmethod(graph_class, (t_method)glist_read,
+    	gensym("read"), A_SYMBOL, A_DEFSYM, A_NULL);
+    class_addmethod(graph_class, (t_method)glist_sort,
+    	gensym("sort"), A_NULL);
 }
