@@ -15,7 +15,6 @@ static t_class *message_class;
 static t_class *gatom_class;
 void canvas_startmotion(t_canvas *x);
 
-
 /* ----------------- the "text" object.  ------------------ */
 
 
@@ -46,7 +45,7 @@ void glist_text(t_glist *gl, t_symbol *s, int argc, t_atom *argv)
     else
     {
     	int xval, yval;
-	vmess((t_pd *)glist_getcanvas(gl), gensym("editmode"), "i", 1);
+	pd_vmess((t_pd *)glist_getcanvas(gl), gensym("editmode"), "i", 1);
     	SETSYMBOL(&at, gensym("comment"));
     	glist_noselect(gl);
     	glist_getnextxy(gl, &xval, &yval);
@@ -109,6 +108,10 @@ static void canvas_objtext(t_glist *gl, int xpos, int ypos, int selected,
     	glist_select(gl, &x->te_g);
     	gobj_activate(&x->te_g, gl, 1);
     }
+    if (pd_class(&x->ob_pd) == vinlet_class)
+    	canvas_resortinlets(glist_getcanvas(gl));
+    if (pd_class(&x->ob_pd) == voutlet_class)
+    	canvas_resortoutlets(glist_getcanvas(gl));
     canvas_unsetcurrent((t_canvas *)gl);
 }
 
@@ -130,7 +133,7 @@ void canvas_obj(t_glist *gl, t_symbol *s, int argc, t_atom *argv)
     {
     	t_binbuf *b = binbuf_new();
     	int xval, yval;
-	vmess(&gl->gl_pd, gensym("editmode"), "i", 1);
+	pd_vmess(&gl->gl_pd, gensym("editmode"), "i", 1);
     	glist_noselect(gl);
     	glist_getnextxy(gl, &xval, &yval);
 	if (xval == 0 && yval == 0) xval = yval = 50;
@@ -228,6 +231,19 @@ static void message_set(t_message *x, t_symbol *s, int argc, t_atom *argv)
     glist_retext(x->m_glist, &x->m_text);
 }
 
+static void message_add2(t_message *x, t_symbol *s, int argc, t_atom *argv)
+{
+    binbuf_add(x->m_text.te_binbuf, argc, argv);
+    glist_retext(x->m_glist, &x->m_text);
+}
+
+static void message_add(t_message *x, t_symbol *s, int argc, t_atom *argv)
+{
+    binbuf_add(x->m_text.te_binbuf, argc, argv);
+    binbuf_addsemi(x->m_text.te_binbuf);
+    glist_retext(x->m_glist, &x->m_text);
+}
+
 static void message_click(t_message *x,
     t_floatarg xpos, t_floatarg ypos, t_floatarg shift,
     	t_floatarg ctrl, t_floatarg alt)
@@ -278,7 +294,7 @@ void canvas_msg(t_glist *gl, t_symbol *s, int argc, t_atom *argv)
     {
     	int xval, yval;
 	glist_getnextxy(gl, &xval, &yval);
-	vmess(&gl->gl_pd, gensym("editmode"), "i", 1);
+	pd_vmess(&gl->gl_pd, gensym("editmode"), "i", 1);
     	glist_noselect(gl);
     	glist_getnextxy(gl, &xval, &yval);
 	if (xval == 0 && yval == 0) xval = yval = 50;
@@ -298,9 +314,11 @@ void canvas_msg(t_glist *gl, t_symbol *s, int argc, t_atom *argv)
 typedef struct _gatom
 {
     t_text a_text;
-    t_atom a_atom;
-    t_glist *a_glist;
-    t_float a_toggle;
+    t_atom a_atom;  	/* this holds the value and the type */
+    t_glist *a_glist;	/* owning glist */
+    t_float a_toggle;	/* value to toggle to */
+    t_float a_draghi;	/* high end of drag range */
+    t_float a_draglo;	/* low end of drag range */
     char a_buf[ATOMBUFSIZE];
     char a_shift;
 } t_gatom;
@@ -308,7 +326,7 @@ typedef struct _gatom
 static void gatom_set(t_gatom *x, t_symbol *s, int argc, t_atom *argv)
 {
     if (!argc) return;
-    else if (x->a_atom.a_type == A_FLOAT)
+    if (x->a_atom.a_type == A_FLOAT)
     	x->a_atom.a_w.w_float = atom_getfloat(argv);
     else if (x->a_atom.a_type == A_SYMBOL)
     	x->a_atom.a_w.w_symbol = atom_getsymbol(argv);
@@ -334,6 +352,18 @@ static void gatom_float(t_gatom *x, t_float f)
     gatom_bang(x);
 }
 
+static void gatom_clipfloat(t_gatom *x, t_float f)
+{
+    if (x->a_draglo != 0 || x->a_draghi != 0)
+    {
+	if (f < x->a_draglo)
+    	    f = x->a_draglo;
+	if (f > x->a_draghi)
+    	    f = x->a_draghi;
+    }
+    gatom_float(x, f);
+}
+
 static void gatom_symbol(t_gatom *x, t_symbol *s)
 {
     t_atom at;
@@ -342,8 +372,9 @@ static void gatom_symbol(t_gatom *x, t_symbol *s)
     gatom_bang(x);
 }
 
-static void gatom_motion(t_gatom *x, t_floatarg dx, t_floatarg dy)
+static void gatom_motion(void *z, t_floatarg dx, t_floatarg dy)
 {
+    t_gatom *x = (t_gatom *)z;
     if (dy == 0) return;
     if (x->a_atom.a_type == A_FLOAT)
     {
@@ -352,7 +383,7 @@ static void gatom_motion(t_gatom *x, t_floatarg dx, t_floatarg dy)
     	    double nval = x->a_atom.a_w.w_float - 0.01 * dy;
     	    double trunc = 0.01 * (floor(100. * nval + 0.5));
     	    if (trunc < nval + 0.0001 && trunc > nval - 0.0001) nval = trunc;
-    	    gatom_float(x, nval);
+    	    gatom_clipfloat(x, nval);
     	}
     	else
     	{
@@ -361,13 +392,14 @@ static void gatom_motion(t_gatom *x, t_floatarg dx, t_floatarg dy)
     	    if (trunc < nval + 0.0001 && trunc > nval - 0.0001) nval = trunc;
     	    trunc = floor(nval + 0.5);
     	    if (trunc < nval + 0.001 && trunc > nval - 0.001) nval = trunc;
-    	    gatom_float(x, nval);
+    	    gatom_clipfloat(x, nval);
     	}
     }
 }
 
-static void gatom_key(t_gatom *x, t_floatarg f)
+static void gatom_key(void *z, t_floatarg f)
 {
+    t_gatom *x = (t_gatom *)z;
     int c = f;
     int l = strlen(x->a_buf);
     t_atom at;
@@ -409,20 +441,44 @@ static void gatom_click(t_gatom *x,
     t_floatarg xpos, t_floatarg ypos, t_floatarg shift, t_floatarg ctrl,
     t_floatarg alt)
 {
-    if (alt)
+    if (x->a_text.te_width == 1)
     {
-    	if (x->a_atom.a_type != A_FLOAT) return;
-    	if (x->a_atom.a_w.w_float != 0)
-    	{
-    	    x->a_toggle = x->a_atom.a_w.w_float;
-    	    gatom_float(x, 0);
-    	    return;
-    	}
-    	else gatom_float(x, x->a_toggle);
+    	if (x->a_atom.a_type == A_FLOAT)
+	    gatom_float(x, (x->a_atom.a_w.w_float == 0));
     }
-    x->a_shift = shift;
-    x->a_buf[0] = 0;
-    glist_grab(x->a_glist, &x->a_text.te_g, xpos, ypos);
+    else
+    {
+	if (alt)
+	{
+    	    if (x->a_atom.a_type != A_FLOAT) return;
+    	    if (x->a_atom.a_w.w_float != 0)
+    	    {
+    		x->a_toggle = x->a_atom.a_w.w_float;
+    		gatom_float(x, 0);
+    		return;
+    	    }
+    	    else gatom_float(x, x->a_toggle);
+	}
+	x->a_shift = shift;
+	x->a_buf[0] = 0;
+	glist_grab(x->a_glist, &x->a_text.te_g, gatom_motion, gatom_key,
+    	    xpos, ypos);
+    }
+}
+
+static void gatom_param(t_gatom *x, t_floatarg width, t_floatarg draglo,
+    t_floatarg draghi)
+{
+    if (draglo >= draghi)
+    	draglo = draghi = 0;
+    x->a_draglo = draglo;
+    x->a_draghi = draghi;
+    if (width < 0)
+    	width = 4;
+    else if (width > 80)
+    	width = 80;
+    x->a_text.te_width = width;
+    glist_retext(x->a_glist, &x->a_text);
 }
 
 void canvas_atom(t_glist *gl, t_atomtype type,
@@ -436,15 +492,19 @@ void canvas_atom(t_glist *gl, t_atomtype type,
     x->a_glist = gl;
     x->a_atom.a_type = type;
     x->a_toggle = 1;
+    x->a_draglo = 0;
+    x->a_draghi = 0;
     if (type == A_FLOAT)
     {
     	x->a_atom.a_w.w_float = 0;
+	x->a_text.te_width = 4;
     	outlet_new(&x->a_text, &s_float);
     	SETFLOAT(&at, 0);
     }
     else
     {
     	x->a_atom.a_w.w_symbol = &s_;
+	x->a_text.te_width = 10;
     	outlet_new(&x->a_text, &s_symbol);
     	SETSYMBOL(&at, &s_);
     }
@@ -453,13 +513,20 @@ void canvas_atom(t_glist *gl, t_atomtype type,
     {
     	x->a_text.te_xpos = atom_getintarg(0, argc, argv);
     	x->a_text.te_ypos = atom_getintarg(1, argc, argv);
+    	x->a_text.te_width = atom_getintarg(2, argc, argv);
+	    /* sanity check because some very old patches have trash in this
+	    field... remove this in 2003 or so: */
+	if (x->a_text.te_width < 0 || x->a_text.te_width > 500)
+	    x->a_text.te_width = 4;
+    	x->a_draglo = atom_getfloatarg(3, argc, argv);
+    	x->a_draghi = atom_getfloatarg(4, argc, argv);
     	glist_add(gl, &x->a_text.te_g);
     }
     else
     {
     	int xval, yval;
 	glist_getnextxy(gl, &xval, &yval);
-	vmess(&gl->gl_pd, gensym("editmode"), "i", 1);
+	pd_vmess(&gl->gl_pd, gensym("editmode"), "i", 1);
     	glist_noselect(gl);
     	glist_getnextxy(gl, &xval, &yval);
 	if (xval == 0 && yval == 0) xval = yval = 50;
@@ -482,6 +549,21 @@ void canvas_symbolatom(t_glist *gl, t_symbol *s, int argc, t_atom *argv)
     canvas_atom(gl, A_SYMBOL, s, argc, argv);
 }
 
+static void gatom_free(t_gatom *x)
+{
+    gfxstub_deleteforkey(x);
+}
+
+static void gatom_properties(t_gobj *z, t_glist *owner)
+{
+    t_gatom *x = (t_gatom *)z;
+    char buf[200];
+    sprintf(buf, "pdtk_gatom_dialog %%s %d %g %g\n",
+	x->a_text.te_width, x->a_draglo, x->a_draghi);
+    gfxstub_new(&x->a_text.te_pd, x, buf);
+}
+
+
 /* -------------------- widget behavior for text objects ------------ */
 
     /* if we're invisible we don't know our size so we just lie about
@@ -492,18 +574,26 @@ static void text_getrect(t_gobj *z, t_glist *glist,
     int *xp1, int *yp1, int *xp2, int *yp2)
 {
     t_text *x = (t_text *)z;
-    int width, height, iscomment = (x->te_type == T_TEXT);
-    if (glist_isvisible(glist))
+    int width, height, iscomment = (x->te_type == T_TEXT),
+    	x1, y1, x2, y2;
+    if (glist->gl_editor)
     {
     	t_rtext *y = glist_findrtext(glist, x);
     	width = rtext_width(y);
     	height = rtext_height(y) - (iscomment << 1);
     }
     else width = height = 10;
-    *xp1 = x->te_xpos;
-    *yp1 = x->te_ypos + iscomment;
-    *xp2 = x->te_xpos + width;
-    *yp2 = x->te_ypos + height;
+    x1 = 
+    x1 = x->te_xpos;
+    y1 = x->te_ypos + iscomment;
+    x2 = x->te_xpos + width;
+    y2 = x->te_ypos + height;
+    if (pd_class(&x->te_pd) == subcanvas_class)
+    	subcanvas_fattenforscalars((t_subcanvas *)x, &x1, &y1, &x2, &y2);
+    *xp1 = x1;
+    *yp1 = y1;
+    *xp2 = x2;
+    *yp2 = y2;
 }
 
 static void text_displace(t_gobj *z, t_glist *glist,
@@ -531,7 +621,6 @@ static void text_select(t_gobj *z, t_glist *glist, int state)
     	rtext_gettag(y), (state? "blue" : "black"));
 }
 
-
 static void text_activate(t_gobj *z, t_glist *glist, int state)
 {
     t_text *x = (t_text *)z;
@@ -560,6 +649,51 @@ static void text_vis(t_gobj *z, t_glist *glist, int vis)
     	text_eraseborder(x, glist, rtext_gettag(y));
     	rtext_free(y);
     }
+    	/* draw any scalars decorating the box; this must happen after
+	text_drawborder() above so the box size is known; see
+	subcanvas_visforscalars(). */
+    if (pd_class(&x->te_pd) == subcanvas_class)
+    	subcanvas_visforscalars((t_subcanvas *)x, glist, vis);
+}
+
+static int text_click(t_gobj *z, struct _glist *glist,
+    int xpix, int ypix, int shift, int alt, int dbl, int doit)
+{
+    t_text *x = (t_text *)z;
+    if (x->te_type == T_OBJECT)
+    {
+    	t_symbol *clicksym = gensym("click");
+    	if (pd_class(&x->te_pd) == subcanvas_class)
+	{
+    	    if (subcanvas_click((t_subcanvas *)x, xpix, ypix, shift,
+	    	alt, dbl, doit))
+		    return (1);
+	}
+	if (zgetfn(&x->te_pd, clicksym))
+	{
+	    if (doit)
+	    	pd_vmess(&x->te_pd, clicksym, "fffff",
+		    (double)xpix, (double)ypix,
+    	    	    	(double)shift, 0, (double)alt);
+	    return (1);
+	}
+	else return (0);
+    }
+    else if (x->te_type == T_ATOM)
+    {
+    	if (doit)
+	    gatom_click((t_gatom *)x, (t_floatarg)xpix, (t_floatarg)ypix,
+	    	(t_floatarg)shift, 0, (t_floatarg)alt);
+	return (1);
+    }
+    else if (x->te_type == T_MESSAGE)
+    {
+    	if (doit)
+	    message_click((t_message *)x, (t_floatarg)xpix, (t_floatarg)ypix,
+	    	(t_floatarg)shift, 0, (t_floatarg)alt);
+	return (1);
+    }
+    else return (0);
 }
 
 int text_isabstraction(t_text *x);
@@ -596,8 +730,9 @@ static void text_save(t_gobj *z, t_binbuf *b)
     	t_atomtype t = ((t_gatom *)x)->a_atom.a_type;
     	t_symbol *sel = (t == A_SYMBOL ? gensym("symbolatom") :
     	    (t == A_FLOAT ? gensym("floatatom") : gensym("intatom")));
-    	binbuf_addv(b, "ssii", gensym("#X"), sel,
-    	    (t_int)x->te_xpos, (t_int)x->te_ypos);
+    	binbuf_addv(b, "ssiiiff", gensym("#X"), sel,
+    	    (t_int)x->te_xpos, (t_int)x->te_ypos, (t_int)x->te_width,
+	    (double)((t_gatom *)x)->a_draglo, (double)((t_gatom *)x)->a_draghi);
         binbuf_addv(b, ";");
     }    	
     else 	
@@ -609,7 +744,7 @@ static void text_save(t_gobj *z, t_binbuf *b)
     }    	
 }
 
-
+    /* this one is for everyone but "gatoms"; it's imposed in m_class.c */
 t_widgetbehavior text_widgetbehavior =
 {
     text_getrect,
@@ -618,15 +753,34 @@ t_widgetbehavior text_widgetbehavior =
     text_activate,
     text_delete,
     text_vis,
-    text_save
+    text_click,
+    text_save,
+    0,
+};
+
+static t_widgetbehavior gatom_widgetbehavior =
+{
+    text_getrect,
+    text_displace,
+    text_select,
+    text_activate,
+    text_delete,
+    text_vis,
+    text_click,
+    text_save,
+    gatom_properties,
 };
 
 /* -------------------- the "text" class  ------------ */
 
 void text_drawborder(t_text *x, t_glist *glist,
-    char *tag, int width, int height, int firsttime)
+    char *tag, int width2, int height2, int firsttime)
 {
     t_object *ob;
+    int x1, y1, x2, y2, width, height;
+    text_getrect(&x->te_g, glist, &x1, &y1, &x2, &y2);
+    width = x2 - x1;
+    height = y2 - y1;
     if (x->te_type == T_OBJECT)
     {
 	if (firsttime)
@@ -739,8 +893,6 @@ void text_drawborder(t_text *x, t_glist *glist,
     	     
    	}
     }
-    
-    x->te_width = width;
 }
 
 void text_eraseborder(t_text *x, t_glist *glist, char *tag)
@@ -769,11 +921,31 @@ void text_setto(t_text *x, t_glist *glist, char *buf, int bufsize)
     if (x->te_type == T_OBJECT)
     {
     	t_binbuf *b = binbuf_new();
+	int natom1, natom2;
+	t_atom *vec1, *vec2;
     	binbuf_text(b, buf, bufsize);
-    	canvas_objtext(glist, x->te_xpos, x->te_ypos, 0, b);
-    	subcanvas_checkloadbang(newest);
-    	glist_delete(glist, &x->te_g);
-    	canvas_restoreconnections(glist_getcanvas(glist));
+	natom1 = binbuf_getnatom(x->te_binbuf);
+	vec1 = binbuf_getvec(x->te_binbuf);
+	natom2 = binbuf_getnatom(b);
+	vec2 = binbuf_getvec(b);
+	    /* special case: if  pd args change just pass the message on. */
+	if (natom1 >= 1 && natom2 >= 1 && vec1[0].a_type == A_SYMBOL
+	    && !strcmp(vec1[0].a_w.w_symbol->s_name, "pd") &&
+	     vec2[0].a_type == A_SYMBOL
+	    && !strcmp(vec2[0].a_w.w_symbol->s_name, "pd"))
+	{
+	    typedmess(&x->te_pd, gensym("rename"), natom2-1, vec2+1);
+	    binbuf_free(x->te_binbuf);
+	    x->te_binbuf = b;
+	}
+	else
+	{
+	    	/* normally, we just destroy the old one and make a new one. */
+    	    canvas_objtext(glist, x->te_xpos, x->te_ypos, 0, b);
+    	    subcanvas_checkloadbang(newest);
+    	    glist_delete(glist, &x->te_g);
+    	    canvas_restoreconnections(glist_getcanvas(glist));
+    	}
     }
     else binbuf_text(x->te_binbuf, buf, bufsize);
 }
@@ -795,6 +967,10 @@ void g_text_setup(void)
     	A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, 0);
     class_addmethod(message_class, (t_method)message_set, gensym("set"),
     	A_GIMME, 0);
+    class_addmethod(message_class, (t_method)message_add, gensym("add"),
+    	A_GIMME, 0);
+    class_addmethod(message_class, (t_method)message_add2, gensym("add2"),
+    	A_GIMME, 0);
 
     messresponder_class = class_new(gensym("messresponder"), 0, 0,
     	sizeof(t_text), CLASS_PD, 0);
@@ -804,18 +980,18 @@ void g_text_setup(void)
     class_addlist(messresponder_class, messresponder_list);
     class_addanything(messresponder_class, messresponder_anything);
 
-    gatom_class = class_new(gensym("gatom"), 0, 0, sizeof(t_gatom),
-    	CLASS_PATCHABLE, 0);
+    gatom_class = class_new(gensym("gatom"), 0, (t_method)gatom_free,
+    	sizeof(t_gatom), CLASS_PATCHABLE, 0);
     class_addbang(gatom_class, gatom_bang);
     class_addfloat(gatom_class, gatom_float);
+    class_addsymbol(gatom_class, gatom_symbol);
     class_addmethod(gatom_class, (t_method)gatom_set, gensym("set"),
     	A_GIMME, 0);
     class_addmethod(gatom_class, (t_method)gatom_click, gensym("click"),
     	A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, 0);
-    class_addmethod(gatom_class, (t_method)gatom_motion, gensym("motion"),
-    	A_FLOAT, A_FLOAT, 0);
-    class_addmethod(gatom_class, (t_method)gatom_key, gensym("key"),
-    	A_FLOAT, 0);
-
+    class_addmethod(gatom_class, (t_method)gatom_param, gensym("param"),
+    	A_DEFFLOAT, A_DEFFLOAT, A_DEFFLOAT, 0);
+    class_setwidget(gatom_class, &gatom_widgetbehavior);
 }
 
+
