@@ -46,7 +46,7 @@ t_gstub *gstub_new(t_glist *gl, t_array *a)
 /* when a "gpointer" is set to point to this stub (so we can later chase
 down the owner) we increase a reference count.  The following routine is called
 whenever a gpointer is unset from pointing here.  If the owner is
-gone and the regcount goes to zero, we can free the object safely. */
+gone and the refcount goes to zero, we can free the gstub safely. */
 
 static void gstub_dis(t_gstub *gs)
 {
@@ -526,15 +526,14 @@ static void *elem_new(t_symbol *templatesym, t_symbol *fieldsym)
 static void elem_float(t_elem *x, t_float f)
 {
     int indx = f, nitems, onset;
-    t_field *field;
     t_symbol *templatesym = x->x_templatesym, *fieldsym = x->x_fieldsym,
-    	*arraytemplate;
-    t_canvas *c = (t_canvas *)pd_findbyclass(templatesym, canvas_class);
-    t_canvas *elemcanvas;
+    	*elemtemplatesym;
+    t_template *template = canvas_gettemplatebyname(templatesym);
+    t_template *elemtemplate;
     t_gpointer *gparent = &x->x_gparent;
     t_word *w;
     t_array *array;
-    int elemsize;
+    int elemsize, type;
     
     if (!gpointer_check(gparent, 0))
     {
@@ -543,25 +542,30 @@ static void elem_float(t_elem *x, t_float f)
     }
     if (gparent->gp_stub->gs_which == GP_ARRAY) w = gparent->gp_un.gp_w;
     else w = gparent->gp_un.gp_scalar->x_vec;
-    if (!c)
+    if (!template)
     {
     	error("element: couldn't find template %s", templatesym->s_name);
     	return;
     }
-    if (!(field = canvas_find_field(c, fieldsym, &onset)))
+    if (!template_find_field(template, fieldsym,
+    	&onset, &type, &elemtemplatesym))
     {
-    	error("element: couldn't find field %s", fieldsym->s_name);
+    	error("element: couldn't find array field %s", fieldsym->s_name);
     	return;
     }
-    if (field_type(field) != DT_ARRAY)
+    if (type != DT_ARRAY)
     {
     	error("element: field %s not of type array", fieldsym->s_name);
     	return;
     }
-    arraytemplate = field_arraytemplate(field);
-    elemcanvas = (t_canvas *)pd_findbyclass(arraytemplate, canvas_class);
+    if (!(elemtemplate = canvas_gettemplatebyname(elemtemplatesym)))
+    {
+    	error("element: couldn't find field template %s",
+	    elemtemplatesym->s_name);
+    	return;
+    }
 
-    elemsize = canvas_template_size(elemcanvas);
+    elemsize = elemtemplate->t_n * sizeof(t_word);
 
     array = *(t_array **)(((char *)w) + onset);
 
@@ -609,28 +613,26 @@ static void *getsize_new(t_symbol *templatesym, t_symbol *fieldsym)
 
 static void getsize_pointer(t_getsize *x, t_gpointer *gp)
 {
-    int nitems, onset;
-    t_field *field;
+    int nitems, onset, type;
     t_symbol *templatesym = x->x_templatesym, *fieldsym = x->x_fieldsym,
-    	*arraytemplate;
-    t_canvas *c = (t_canvas *)pd_findbyclass(templatesym, canvas_class);
-    t_canvas *elemcanvas;
+    	*elemtemplatesym;
+    t_template *template = canvas_gettemplatebyname(templatesym);
     t_word *w;
-    t_atom at;
     t_array *array;
     int elemsize;
     t_gstub *gs = gp->gp_stub;
-    if (!c)
+    if (!template)
     {
     	error("getsize: couldn't find template %s", templatesym->s_name);
     	return;
     }
-    if (!(field = canvas_find_field(c, fieldsym, &onset)))
+    if (!template_find_field(template, fieldsym,
+    	&onset, &type, &elemtemplatesym))
     {
-    	error("getsize: couldn't find field %s", fieldsym->s_name);
+    	error("getsize: couldn't find array field %s", fieldsym->s_name);
     	return;
     }
-    if (field_type(field) != DT_ARRAY)
+    if (type != DT_ARRAY)
     {
     	error("getsize: field %s not of type array", fieldsym->s_name);
     	return;
@@ -680,12 +682,11 @@ static void *setsize_new(t_symbol *templatesym, t_symbol *fieldsym,
 
 static void setsize_float(t_setsize *x, t_float f)
 {
-    int nitems, onset;
-    t_field *field;
+    int nitems, onset, type;
     t_symbol *templatesym = x->x_templatesym, *fieldsym = x->x_fieldsym,
-    	*arraytemplate;
-    t_canvas *c = (t_canvas *)pd_findbyclass(templatesym, canvas_class);
-    t_canvas *elemcanvas;
+    	*elemtemplatesym;
+    t_template *template = canvas_gettemplatebyname(templatesym);
+    t_template *elemtemplate;
     t_word *w;
     t_atom at;
     t_array *array;
@@ -701,29 +702,34 @@ static void setsize_float(t_setsize *x, t_float f)
     if (gs->gs_which == GP_ARRAY) w = gp->gp_un.gp_w;
     else w = gp->gp_un.gp_scalar->x_vec;
 
-    if (!c)
+    if (!template)
     {
     	error("setsize: couldn't find template %s", templatesym->s_name);
     	return;
     }
-    if (!(field = canvas_find_field(c, fieldsym, &onset)))
+    if (!template_find_field(template, fieldsym,
+    	&onset, &type, &elemtemplatesym))
     {
-    	error("setsize: couldn't find field %s", fieldsym->s_name);
+    	error("setsize: couldn't find array field %s", fieldsym->s_name);
     	return;
     }
-    if (field_type(field) != DT_ARRAY)
+    if (type != DT_ARRAY)
     {
     	error("setsize: field %s not of type array", fieldsym->s_name);
     	return;
     }
 
+    if (!(elemtemplate = canvas_gettemplatebyname(elemtemplatesym)))
+    {
+    	error("element: couldn't find field template %s",
+	    elemtemplatesym->s_name);
+    	return;
+    }
+
+    elemsize = elemtemplate->t_n * sizeof(t_word);
+
     array = *(t_array **)(((char *)w) + onset);
 
-    arraytemplate = field_arraytemplate(field);
-    elemcanvas = (t_canvas *)pd_findbyclass(arraytemplate, canvas_class);
-
-    elemsize = canvas_template_size(elemcanvas);
-
     if (elemsize != array->a_elemsize) bug("setsize_gpointer");
 
     nitems = array->a_n;
@@ -752,19 +758,15 @@ static void setsize_float(t_setsize *x, t_float f)
     array->a_n = newsize;
     if (newsize > nitems)
     {
-    	t_datatype *datatypes;
-    	int ntypes;
     	char *newelem = ((char *)array->a_vec) + nitems * elemsize;
     	int i = 0, nnew = newsize - nitems;
     	
-    	datatypes = canvas_getdatatypes(elemcanvas, &ntypes);
     	while (nnew--)
     	{
-    	    word_restore((t_word *)newelem, datatypes, ntypes, 0, 0, gp);
+    	    word_init((t_word *)newelem, elemtemplate, gp);
     	    newelem += elemsize;
     	    /* post("new %x %x, ntypes %d", newelem, *(int *)newelem, ntypes); */
     	}
-    	freebytes(datatypes, ntypes * sizeof(*datatypes));
     }
 
     	/* redraw again. */
@@ -873,7 +875,12 @@ static void append_float(t_append *x, t_float f)
     if (!nitems) return;
     x->x_variables[0].gv_f = f;
 
-    sc = scalar_new(glist, templatesym, 0, 0);
+    sc = scalar_new(glist, templatesym);
+    if (!sc)
+    {
+    	error("%s: couldn't create scalar", templatesym->s_name);
+    	return;
+    }
     oldsc = gp->gp_un.gp_scalar;
     
     if (oldsc)
@@ -938,16 +945,15 @@ static void *sublist_new(t_symbol *templatesym, t_symbol *fieldsym)
 
 static void sublist_pointer(t_sublist *x, t_gpointer *gp)
 {
-    t_symbol *templatesym = x->x_templatesym;
-    t_canvas *c = (t_canvas *)pd_findbyclass(templatesym, canvas_class);
+    t_symbol *templatesym = x->x_templatesym, *dummy;
+    t_template *template = canvas_gettemplatebyname(templatesym);
     t_gstub *gs = gp->gp_stub;
     t_word *vec; 
     t_getvariable *vp;
-    t_field *field;
-    int onset;
+    int onset, type;
     t_word *w;
 
-    if (!c)
+    if (!template)
     {
     	error("sublist: couldn't find template %s", templatesym->s_name);
     	return;
@@ -957,12 +963,13 @@ static void sublist_pointer(t_sublist *x, t_gpointer *gp)
     	error("sublist: empty pointer");
     	return;
     }
-    if (!(field = canvas_find_field(c, x->x_fieldsym, &onset)))
+    if (!template_find_field(template, x->x_fieldsym,
+    	&onset, &type, &dummy))
     {
     	error("sublist: couldn't find field %s", x->x_fieldsym->s_name);
     	return;
     }
-    if (field_type(field) != DT_LIST)
+    if (type != DT_LIST)
     {
     	error("sublist: field %s not of type list", x->x_fieldsym->s_name);
     	return;
