@@ -173,6 +173,7 @@ void binbuf_text(t_binbuf *x, char *text, size_t size)
     x->b_n = natom;
 }
 
+    /* convert a binbuf to text; no null termination. */
 void binbuf_gettext(t_binbuf *x, char **bufp, int *lengthp)
 {
     char *buf = getbytes(0), *newbuf;
@@ -310,6 +311,13 @@ void binbuf_addbinbuf(t_binbuf *x, t_binbuf *y)
     binbuf_add(x, z->b_n, z->b_vec);
 }
 
+void binbuf_addsemi(t_binbuf *x)
+{
+    t_atom a;
+    SETSEMI(&a);
+    binbuf_add(x, 1, &a);
+}
+
 /* Supply atoms to a binbuf from a message, making the opposite changes
 from binbuf_addbinbuf.  The symbol ";" goes to a semicolon, etc. */
 
@@ -335,11 +343,20 @@ void binbuf_restore(t_binbuf *x, int argc, t_atom *argv)
     	    else if (!strcmp(str, ",")) SETCOMMA(ap);
     	    else if (str[0] == '$' && str[1] >= '0' && str[1] <= '9')
     	    {
-	    	    /* LATER detect DOLLSYM */
-    	    	int dollar = 0;
-    	    	sscanf(argv->a_w.w_symbol->s_name + 1, "%d", &dollar);
-    	    	SETDOLLAR(ap, dollar);
-    	    }
+	    	int dollsym = 0;
+	    	char *str2;
+		for (str2 = str + 2; *str2; str2++)
+		    if (*str2 < '0' || *str2 > '9')
+		    	dollsym = 1;
+    	    	if (dollsym)
+		    SETDOLLSYM(ap, gensym(str + 1));
+		else
+		{
+		    int dollar = 0;
+    	    	    sscanf(argv->a_w.w_symbol->s_name + 1, "%d", &dollar);
+    	    	    SETDOLLAR(ap, dollar);
+    	    	}
+	    }
     	    else *ap = *argv;
     	    argv++;
     	}
@@ -351,16 +368,23 @@ void binbuf_restore(t_binbuf *x, int argc, t_atom *argv)
 
 #define MSTACKSIZE 2048
 
-static int realizedollsym(t_atom *at, char *name, int argc, t_atom *argv)
-{
-    error("dollsym: not implemented yet");
-    return(1);
-}
-
 void binbuf_print(t_binbuf *x)
 {
-    startpost("");
-    postatom(x->b_n, x->b_vec);
+    int i, startedpost = 0, newline = 1;
+    for (i = 0; i < x->b_n; i++)
+    {
+    	if (newline)
+	{
+	    if (startedpost) endpost();
+	    startpost("");
+	    startedpost = 1;
+    	}
+	postatom(1, x->b_vec + i);
+	if (x->b_vec[i].a_type == A_SEMI)
+	    newline = 1;
+	else newline = 0; 
+    }
+    if (startedpost) endpost();
 }
 
 int binbuf_getnatom(t_binbuf *x)
@@ -373,6 +397,29 @@ t_atom *binbuf_getvec(t_binbuf *x)
     return (x->b_vec);
 }
 
+int canvas_getdollarzero( void);
+
+static t_symbol *realizedollsym(t_symbol *s, int ac, t_atom *av, int tonew)
+{
+    int argno = atol(s->s_name), lastnum;
+    char buf[MAXPDSTRING], c, *sp;
+    for (lastnum = 0, sp = s->s_name; ((c = *sp) && c >= '0' && c <= '9');
+    	sp++, lastnum++)
+    if (!c || argno < 0 || argno > ac)
+    {
+    	if (!tonew)
+    	    return (0);
+	else sprintf(buf, "$%d", argno);
+    }
+    else if (argno == 0)
+    	sprintf(buf, "%d", canvas_getdollarzero());
+    else
+    	atom_string(av+(argno-1), buf, MAXPDSTRING/2-1);
+    strncat(buf, sp, MAXPDSTRING/2-1);
+    return (gensym(buf));
+
+}
+
 void binbuf_eval(t_binbuf *x, t_pd *target, int argc, t_atom *argv)
 {
     static t_atom mstack[MSTACKSIZE], *msp = mstack, *ems = mstack+MSTACKSIZE;
@@ -393,7 +440,7 @@ void binbuf_eval(t_binbuf *x, t_pd *target, int argc, t_atom *argv)
 	    if (!ac) break;
 	    if (at->a_type == A_DOLLAR)
 	    {
-	    	if (at->a_w.w_index < 0 || at->a_w.w_index > argc)
+	    	if (at->a_w.w_index <= 0 || at->a_w.w_index > argc)
 	    	{
 		    error("$%d: not enough arguments supplied",
 			    at->a_w.w_index);
@@ -407,6 +454,15 @@ void binbuf_eval(t_binbuf *x, t_pd *target, int argc, t_atom *argv)
 		}
 		else s = argv[at->a_w.w_index-1].a_w.w_symbol;
 	    }
+	    else if (at->a_type == A_DOLLSYM)
+	    {
+	    	if (!(s = realizedollsym(at->a_w.w_symbol, argc, argv, 0)))
+		{
+		    error("$%s: not enough arguments supplied",
+		    	at->a_w.w_symbol->s_name);
+		    goto cleanup;
+	    	}
+	    }
 	    else s = atom_getsymbol(at);
 	    if (!(target = s->s_thing))
 	    {
@@ -428,6 +484,7 @@ void binbuf_eval(t_binbuf *x, t_pd *target, int argc, t_atom *argv)
 	nexttarget = target;
 	while (1)
 	{
+	    t_symbol *s9;
 	    if (!ac) goto gotmess;
 	    if (msp >= ems)
 	    {
@@ -445,22 +502,28 @@ void binbuf_eval(t_binbuf *x, t_pd *target, int argc, t_atom *argv)
 		*msp = *at;
 		break;
 	    case A_DOLLAR:
-		if (at->a_w.w_index <= 0 || at->a_w.w_index > argc)
+		if (at->a_w.w_index > 0 && at->a_w.w_index <= argc)
+		    *msp = argv[at->a_w.w_index-1];
+	    	else if (at->a_w.w_index == 0)
+		    SETFLOAT(msp, canvas_getdollarzero());
+		else
 		{
 		    if (target == s__N.s_thing)
 		    	SETFLOAT(msp, 0);
 		    else
 		    {
-		    	error("$%d: not enough arguments supplied",
+		    	error("$%d: argument number out of range",
 			    at->a_w.w_index);
 		    	SETFLOAT(msp, 0);
 		    }
 		}
-		else *msp = argv[at->a_w.w_index-1];
 		break;
 	    case A_DOLLSYM:
-		if (realizedollsym(msp, at->a_w.w_symbol->s_name, argc, argv))
+	    	s9 = realizedollsym(at->a_w.w_symbol, argc, argv,
+		    target == s__N.s_thing);
+		if (!s9)
 		    goto broken;
+	    	SETSYMBOL(msp, s9);
 		break;
 	    default:
 		bug("bad item in binbuf");
@@ -568,26 +631,50 @@ int binbuf_read(t_binbuf *b, char *filename, char *dirname, int crflag)
     return (0);
 }
 
+int binbuf_read_via_path(t_binbuf *b, char *filename, char *dirname,
+    int crflag)
+{
+    int filedesc;
+    char buf[MAXPDSTRING], *bufptr;
+    if ((filedesc = open_via_path(
+    	dirname, filename, "", buf, &bufptr, MAXPDSTRING, 0)) < 0)
+    {
+    	error("%s: can't open", filename);
+    	return (1);
+    }
+    else close (filedesc);
+    if (binbuf_read(b, bufptr, buf, crflag))
+	return (1);
+    else return (0);
+}
+
 #define WBUFSIZE 4096
+static t_binbuf *binbuf_convert(t_binbuf *oldb, int maxtopd);
 
     /* write a binbuf to a text file.  If "crflag" is set we suppress
     semicolons. */
 int binbuf_write(t_binbuf *x, char *filename, char *dir, int crflag)
 {
-    FILE *f;
+    FILE *f = 0;
     char sbuf[WBUFSIZE], fbuf[MAXPDSTRING], *bp = sbuf, *ep = sbuf + WBUFSIZE;
     t_atom *ap;
-    int indx;
-
+    int indx, deleteit = 0;
+    
     fbuf[0] = 0;
     if (*dir)
     	strcat(fbuf, dir), strcat(fbuf, "/");
     strcat(fbuf, filename);
+    if (!strcmp(filename + strlen(filename) - 4, ".pat"))
+    {
+    	x = binbuf_convert(x, 0);
+	deleteit = 1;
+    }
+    
     if (!(f = binbuf_dofopen(fbuf, "w")))
     {
     	fprintf(stderr, "open: ");
     	sys_unixerror(fbuf);
-    	return (1);
+    	goto fail;
     }
     for (ap = x->b_vec, indx = x->b_n; indx--; ap++)
     {
@@ -602,8 +689,7 @@ int binbuf_write(t_binbuf *x, char *filename, char *dir, int crflag)
     	    if (fwrite(sbuf, bp-sbuf, 1, f) < 1)
     	    {
     	    	sys_unixerror(fbuf);
-    	    	fclose(f);
-    	    	return(1);
+    	    	goto fail;
     	    }
     	    bp = sbuf;
     	}
@@ -620,14 +706,25 @@ int binbuf_write(t_binbuf *x, char *filename, char *dir, int crflag)
     if (fwrite(sbuf, bp-sbuf, 1, f) < 1)
     {
     	sys_unixerror(fbuf);
-    	fclose(f);
-    	return(1);
+    	goto fail;
     }
+    if (deleteit)
+    	binbuf_free(x);
     fclose(f);
     return (0);
+fail:
+    if (deleteit)
+    	binbuf_free(x);
+    if (f)
+    	fclose(f);
+    return (1);
 }
 
-/* The following routine attempts to convert from max to pd. */
+/* The following routine attempts to convert from max to pd or back.  The
+max to pd direction is working OK but you will need to make lots of 
+abstractions for objects like "gate" which don't exist in Pd.  conversiio
+from Pd to Max hasn't been tested for patches with subpatches yet!  */
+
 #define MAXSTACK 1000
 
 #define ISSYMBOL(a, b) ((a)->a_type == A_SYMBOL && \
@@ -640,6 +737,8 @@ static t_binbuf *binbuf_convert(t_binbuf *oldb, int maxtopd)
     t_int n = oldb->b_n, nextindex, stackdepth = 0, stack[MAXSTACK],
     	nobj = 0, i;
     t_atom outmess[MAXSTACK], *nextmess;
+    if (!maxtopd)
+    	binbuf_addv(newb, "ss;", gensym("max"), gensym("v2"));
     for (nextindex = 0; nextindex < n; )
     {
     	int endmess, natom;
@@ -660,22 +759,26 @@ static t_binbuf *binbuf_convert(t_binbuf *oldb, int maxtopd)
 	nextmess = vec + nextindex;
 	first = nextmess->a_w.w_symbol->s_name;
 	second = (nextmess+1)->a_w.w_symbol->s_name;
-	for (i = 0; i < natom; i++)
-	{
-	    if (nextmess[i].a_type == A_DOLLAR)
-	    {
-	    	char buf[100];
-		sprintf(buf, "$%d", nextmess[i].a_w.w_index);
-		SETSYMBOL(nextmess+i, gensym(buf));
-	    }
-	    else if (nextmess[i].a_type == A_DOLLSYM)
-	    {
-	    	nextmess[i].a_type = A_DOLLAR;
-		nextmess[i].a_w.w_index = 0;
-	    }
-	}
 	if (maxtopd)
 	{
+	    	/* case 1: importing a ".pat" file into Pd. */
+		
+		/* dollar signs in file translate to symbols */
+	    for (i = 0; i < natom; i++)
+	    {
+		if (nextmess[i].a_type == A_DOLLAR)
+		{
+	    	    char buf[100];
+		    sprintf(buf, "$%d", nextmess[i].a_w.w_index);
+		    SETSYMBOL(nextmess+i, gensym(buf));
+		}
+		else if (nextmess[i].a_type == A_DOLLSYM)
+		{
+	    	    char buf[100];
+		    sprintf(buf, "$%s", nextmess[i].a_w.w_symbol->s_name);
+		    SETSYMBOL(nextmess+i, gensym(buf));
+		}
+	    }
 	    if (!strcmp(first, "#N"))
 	    {
 	    	if (!strcmp(second, "vpatcher"))
@@ -692,15 +795,18 @@ static t_binbuf *binbuf_convert(t_binbuf *oldb, int maxtopd)
 		    	gensym("#N"), gensym("canvas"),
 			    atom_getfloatarg(2, natom, nextmess),
 			    atom_getfloatarg(3, natom, nextmess),
-			    atom_getfloatarg(4, natom, nextmess),
-			    atom_getfloatarg(5, natom, nextmess),
+			    atom_getfloatarg(4, natom, nextmess) -
+			    	atom_getfloatarg(2, natom, nextmess),
+			    atom_getfloatarg(5, natom, nextmess) -
+			    	atom_getfloatarg(3, natom, nextmess),
 			    12.);
 	    	}
 	    }
 	    if (!strcmp(first, "#P"))
 	    {
 	    	if (natom >= 7 && !strcmp(second, "newobj")
-		    && (ISSYMBOL (&nextmess[6], "patcher")))
+		    && (ISSYMBOL(&nextmess[6], "patcher") ||
+		    	ISSYMBOL(&nextmess[6], "p")))
 		{
 		    binbuf_addv(newb, "ssffss;",
 		    	gensym("#X"), gensym("restore"),
@@ -783,7 +889,8 @@ static t_binbuf *binbuf_convert(t_binbuf *oldb, int maxtopd)
 			gensym((natom > 5 ? "outlet~" : "outlet"))); 
 		    nobj++;
 		}
-		else if (!strcmp(second, "connect"))
+		else if (!strcmp(second, "connect")||
+		    !strcmp(second, "fasten"))
 		{
 		    binbuf_addv(newb, "ssffff;",
 			gensym("#X"), gensym("connect"),
@@ -794,9 +901,130 @@ static t_binbuf *binbuf_convert(t_binbuf *oldb, int maxtopd)
 		}
 	    }
 	}
+	else	    /* Pd to Max */
+	{
+	    if (!strcmp(first, "#N"))
+	    {
+	    	if (!strcmp(second, "canvas"))
+		{
+		    if (stackdepth >= MAXSTACK)
+		    {
+		    	post("too many embedded patches");
+			return (newb);
+		    }
+		    stack[stackdepth] = nobj;
+		    stackdepth++;
+		    nobj = 0;
+		    binbuf_addv(newb, "ssffff;", 
+		    	gensym("#N"), gensym("vpatcher"),
+			    atom_getfloatarg(2, natom, nextmess),
+			    atom_getfloatarg(3, natom, nextmess),
+			    atom_getfloatarg(4, natom, nextmess),
+			    atom_getfloatarg(5, natom, nextmess));
+	    	}
+	    }
+	    if (!strcmp(first, "#X"))
+	    {
+	    	if (natom >= 5 && !strcmp(second, "restore")
+		    && (ISSYMBOL (&nextmess[4], "pd")))
+		{
+		    binbuf_addv(newb, "ss;", gensym("#P"), gensym("pop"));
+		    binbuf_addv(newb, "ssffffss;",
+		    	gensym("#P"), gensym("newobj"),
+			atom_getfloatarg(2, natom, nextmess),
+			atom_getfloatarg(3, natom, nextmess), 50., 1.,
+			gensym("patcher"),
+			    atom_getsymbolarg(5, natom, nextmess));
+		    if (stackdepth) stackdepth--;
+		    nobj = stack[stackdepth];
+		    nobj++;
+		}
+		else if (!strcmp(second, "obj"))
+		{
+		    t_symbol *classname =
+		    	atom_getsymbolarg(4, natom, nextmess);
+		    if (classname == gensym("inlet"))
+		    	binbuf_addv(newb, "ssfff;", gensym("#P"),
+			    gensym("inlet"),
+			    atom_getfloatarg(2, natom, nextmess),
+			    atom_getfloatarg(3, natom, nextmess),
+			    15.);
+		    else if (classname == gensym("inlet~"))
+		    	binbuf_addv(newb, "ssffff;", gensym("#P"),
+			    gensym("inlet"),
+			    atom_getfloatarg(2, natom, nextmess),
+			    atom_getfloatarg(3, natom, nextmess),
+			    15., 1.);
+		    else if (classname == gensym("outlet"))
+		    	binbuf_addv(newb, "ssfff;", gensym("#P"),
+			    gensym("outlet"),
+			    atom_getfloatarg(2, natom, nextmess),
+			    atom_getfloatarg(3, natom, nextmess),
+			    15.);
+		    else if (classname == gensym("outlet~"))
+		    	binbuf_addv(newb, "ssffff;", gensym("#P"),
+			    gensym("outlet"),
+			    atom_getfloatarg(2, natom, nextmess),
+			    atom_getfloatarg(3, natom, nextmess),
+			    15., 1.);
+		    else
+		    {
+	    	    	SETSYMBOL(outmess, gensym("#P"));
+	    	    	SETSYMBOL(outmess + 1, gensym("newex"));
+		    	outmess[2] = nextmess[2];
+		    	outmess[3] = nextmess[3];
+			SETFLOAT(outmess + 4, 50);
+			SETFLOAT(outmess + 5, 1);
+		    	for (i = 4; i < natom; i++)
+			    outmess[i+2] = nextmess[i];
+	    	    	SETSEMI(outmess + natom + 2);
+		    	binbuf_add(newb, natom + 3, outmess);
+		    }
+		    nobj++;
+		
+		}
+		else if (!strcmp(second, "msg") || 
+	    	    !strcmp(second, "text"))
+		{
+	    	    SETSYMBOL(outmess, gensym("#P"));
+	    	    SETSYMBOL(outmess + 1, gensym(
+			(strcmp(second, "msg") ? "message" : "comment")));
+		    outmess[2] = nextmess[2];
+		    outmess[3] = nextmess[3];
+		    SETFLOAT(outmess + 4, 50);
+		    SETFLOAT(outmess + 5, 1);
+		    for (i = 4; i < natom; i++)
+			outmess[i+2] = nextmess[i];
+	    	    SETSEMI(outmess + natom + 2);
+		    binbuf_add(newb, natom + 3, outmess);
+		    nobj++;
+		}
+		else if (!strcmp(second, "floatatom"))
+		{
+		    binbuf_addv(newb, "ssfff;",
+			gensym("#P"), gensym("flonum"),
+			atom_getfloatarg(2, natom, nextmess),
+			atom_getfloatarg(3, natom, nextmess), 35);
+		    nobj++;
+		}
+		else if (!strcmp(second, "connect"))
+		{
+		    binbuf_addv(newb, "ssffff;",
+			gensym("#P"), gensym("connect"),
+			nobj - atom_getfloatarg(2, natom, nextmess) - 1,
+			atom_getfloatarg(3, natom, nextmess),
+			nobj - atom_getfloatarg(4, natom, nextmess) - 1,
+			atom_getfloatarg(5, natom, nextmess)); 
+		}
+	    }
+	}
 	nextindex = endmess + 1;
     }
+    if (!maxtopd)
+    	binbuf_addv(newb, "ss;", gensym("#P"), gensym("pop"));
+#if 1
     binbuf_write(newb, "import-result.pd", "/tmp", 0);
+#endif
     return (newb);
 }
 
@@ -815,8 +1043,10 @@ int binbuf_match(t_binbuf *inbuf, t_binbuf *searchbuf)
 		    ||
 	    	a1->a_type == A_FLOAT && a1->a_w.w_float != a2->a_w.w_float
 		    ||
-	    	a1->a_type == A_DOLLAR && a1->a_w.w_index != a2->a_w.w_index)
-	    	goto nomatch;
+	    	a1->a_type == A_DOLLAR && a1->a_w.w_index != a2->a_w.w_index
+		    ||
+	    	a1->a_type == A_DOLLSYM && a1->a_w.w_symbol != a2->a_w.w_symbol)
+	    	    goto nomatch;
 	}
 	return (1);
     nomatch: ;
