@@ -8,11 +8,15 @@
 #ifdef UNIX
 #include <unistd.h>
 #endif
+#ifdef NT
+#include <io.h>
+#endif
 
 #include <stdarg.h>
 #include <string.h>
 
 static t_class *newclass;
+static t_symbol *class_loadsym;     /* name under which an extern is invoked */
 static void pd_defaultfloat(t_pd *x, t_float f);
 static void pd_defaultlist(t_pd *x, t_symbol *s, int argc, t_atom *argv);
 
@@ -134,8 +138,22 @@ t_class *class_new(t_symbol *s, t_newmethod newmethod, t_method freemethod,
     }
     va_end(ap);
     if (newclass && newmethod)
+    {
+    	    /* add a "new" method by the name specified by the object */
     	class_addmethod(newclass, (t_method)newmethod, s,
     	    vec[0], vec[1], vec[2], vec[3], vec[4], vec[5]);
+	if (class_loadsym)
+	{
+	    	/* if we're loading an extern it might have been invoked by a
+		longer file name; in this case, make this an admissible name
+		too. */
+    	    char *loadstring = class_loadsym->s_name,
+	    	l1 = strlen(s->s_name), l2 = strlen(loadstring);
+	    if (l2 > l1 && !strcmp(s->s_name, loadstring + (l2 - l1)))
+	    	class_addmethod(newclass, (t_method)newmethod, class_loadsym,
+    	    	    vec[0], vec[1], vec[2], vec[3], vec[4], vec[5]);
+	}
+    }
     c = (t_class *)t_getbytes(sizeof(*c));
     c->c_name = c->c_helpname = s;
     c->c_size = size;
@@ -154,6 +172,7 @@ t_class *class_new(t_symbol *s, t_newmethod newmethod, t_method freemethod,
     c->c_patchable = (typeflag == CLASS_PATCHABLE);
     c->c_gobj = (typeflag >= CLASS_GOBJ);
     c->c_drawcommand = 0;
+    c->c_floatsignalin = 0;
 #if 0 
     post("class: %s", c->c_name->s_name);
 #endif
@@ -200,6 +219,15 @@ void class_addmethod(t_class *c, t_method fn, t_symbol *sel,
     int nargs;
     
     va_start(ap, arg1);
+    	/* "signal" method specifies that we take audio signals but
+	that we don't want automatic float to signal conversion.  This
+	is obsolete; you should now use the CLASS_MAINSIGNALIN macro. */
+    if (sel == &s_signal)
+    {
+    	if (c->c_floatsignalin)
+	    post("warning: signal method overrides class_mainsignalin");
+    	c->c_floatsignalin = -1;
+    }
     	/* check for special cases.  "Pointer" is missing here so that
     	"#N"'s pointer method can be typechecked differently.  */
     if (sel == &s_bang)
@@ -242,6 +270,9 @@ void class_addmethod(t_class *c, t_method fn, t_symbol *sel,
     	    m->me_arg[nargs++] = argtype;
     	    argtype = va_arg(ap, t_atomtype);
 	}
+	if (argtype != A_NULL)
+	    error("%s_%s: only 5 arguments are typecheckable; use A_GIMME",
+	    	c->c_name->s_name, sel->s_name);
 	va_end(ap);
 	m->me_arg[nargs] = A_NULL;
     }
@@ -322,6 +353,28 @@ int class_isdrawcommand(t_class *c)
     return (c->c_drawcommand);
 }
 
+static void pd_floatforsignal(t_pd *x, t_float f)
+{
+    int offset = (*x)->c_floatsignalin;
+    if (offset > 0)
+    	*(t_sample *)(((char *)x) + offset) = f;
+    else
+    	pd_error(x, "%s: float unexpected for signal input",
+	    (*x)->c_name->s_name);
+}
+
+void class_domainsignalin(t_class *c, int onset)
+{
+    if (onset <= 0) onset = -1;
+    else
+    {
+    	if (c->c_floatmethod != pd_defaultfloat)
+	    post("warning: %s: float method overwritten", c->c_name->s_name);
+	c->c_floatmethod = (t_floatmethod)pd_floatforsignal;
+    }
+    c->c_floatsignalin = onset;
+}
+
 /* ---------------- the symbol table ------------------------ */
 
 #define HASHSIZE 1024
@@ -381,7 +434,10 @@ void canvas_popabstraction(t_canvas *x);
 extern t_pd *newest;
 
 t_symbol* pathsearch(t_symbol *s,char* ext);
+int pd_setloadingabstraction(t_symbol *sym);
 
+    /* this routine is called when a new "object" is requested whose class Pd
+    doesn't know.  Pd tries to load it as an extern, then as an absteaction. */
 void new_anything(void *dummy, t_symbol *s, int argc, t_atom *argv)
 {
     t_pd *current;
@@ -390,6 +446,7 @@ void new_anything(void *dummy, t_symbol *s, int argc, t_atom *argv)
     char dirbuf[MAXPDSTRING], *nameptr;
     if (tryingalready) return;
     newest = 0;
+    class_loadsym = s;
     if (sys_load_lib(dir->s_name, s->s_name))
     {
     	tryingalready = 1;
@@ -397,15 +454,20 @@ void new_anything(void *dummy, t_symbol *s, int argc, t_atom *argv)
     	tryingalready = 0;
     	return;
     }
+    class_loadsym = 0;
     current = s__X.s_thing;
     canvas_setargs(argc, argv);
-    if (fd = open_via_path(dir->s_name, s->s_name, ".pd",
-    	dirbuf, &nameptr, MAXPDSTRING, 0) >= 0)
+    if ((fd = open_via_path(dir->s_name, s->s_name, ".pd",
+    	dirbuf, &nameptr, MAXPDSTRING, 0)) >= 0)
     {
     	close (fd);
-    	binbuf_evalfile(gensym(nameptr), gensym(dirbuf));
-    	if (s__X.s_thing != current)
-    	    canvas_popabstraction((t_canvas *)(s__X.s_thing));
+	if (!pd_setloadingabstraction(s))
+	{
+    	    binbuf_evalfile(gensym(nameptr), gensym(dirbuf));
+    	    if (s__X.s_thing != current)
+    	    	canvas_popabstraction((t_canvas *)(s__X.s_thing));
+    	}
+	else error("%s: can't load abstraction within itself\n", s->s_name);
     }
     else newest = 0;
 }
@@ -419,10 +481,12 @@ t_symbol  s_anything =	{"anything", 0, 0};
 t_symbol  s_signal =	{"signal", 0, 0};
 t_symbol  s__N =	{"#N", 0, 0};
 t_symbol  s__X =	{"#X", 0, 0};
+t_symbol  s_x =     	{"x", 0, 0};
+t_symbol  s_y =     	{"y", 0, 0};
 t_symbol  s_ = 	    	{"", 0, 0};
 
 static t_symbol *symlist[] = { &s_pointer, &s_float, &s_symbol, &s_bang,
-    &s_list, &s_anything, &s_signal, &s__N, &s__X, &s_};
+    &s_list, &s_anything, &s_signal, &s__N, &s__X, &s_x, &s_y, &s_};
 
 void mess_init(void)
 {
@@ -598,7 +662,7 @@ badarg:
     	s->s_name, c->c_name->s_name);
 }
 
-void vmess(t_pd *x, t_symbol *sel, char *fmt, ...)
+void pd_vmess(t_pd *x, t_symbol *sel, char *fmt, ...)
 {
     va_list ap;
     t_atom arg[MAXPDARG], *at =arg;
