@@ -11,24 +11,28 @@
 #include <stdio.h>
 #endif
 
+/* #define DEBUGMEM */
+#ifdef DEBUGMEM
+static int totalmem = 0;
+#endif
+
 void *getbytes(size_t nbytes)
 {
     void *ret;
     if (nbytes < 1) nbytes = 1;
-    ret = (void *)malloc(nbytes);
+    ret = (void *)calloc(nbytes, 1);
 #ifdef LOUD
-    fprintf(stderr, "new  %x %d\n", ret, nbytes);
+    fprintf(stderr, "new  %x %d\n", (int)ret, nbytes);
 #endif /* LOUD */
+#ifdef DEBUGMEM
+    totalmem += nbytes;
+#endif
     return (ret);
 }
 
-void *getzbytes(size_t nbytes)	/* get bytes, zeroed out */
+void *getzbytes(size_t nbytes)	/* obsolete name */
 {
-    void *ret;
-    if (nbytes < 1) nbytes = 1;
-    ret = getbytes(nbytes);
-    if (ret) memset(ret, 0, nbytes);
-    return (ret);
+    return (getbytes(nbytes));
 }
 
 void *copybytes(void *src, size_t nbytes)
@@ -44,18 +48,37 @@ void *resizebytes(void *old, size_t oldsize, size_t newsize)
 {
     void *ret;
     if (newsize < 1) newsize = 1;
+    if (oldsize < 1) oldsize = 1;
     ret = (void *)realloc((char *)old, newsize);
+    if (newsize > oldsize && ret)
+    	memset(((char *)ret) + oldsize, 0, newsize - oldsize);
 #ifdef LOUD
-    fprintf(stderr, "resize %x %d --> %x %d\n", old, oldsize, ret, newsize);
+    fprintf(stderr, "resize %x %d --> %x %d\n", (int)old, oldsize, (int)ret, newsize);
 #endif /* LOUD */
+#ifdef DEBUGMEM
+    totalmem += (newsize - oldsize);
+#endif
     return (ret);
 }
 
 void freebytes(void *fatso, size_t nbytes)
 {
+    if (nbytes == 0)
+    	nbytes = 1;
 #ifdef LOUD
-    fprintf(stderr, "free %x %d\n", fatso, nbytes);
+    fprintf(stderr, "free %x %d\n", (int)fatso, nbytes);
 #endif /* LOUD */
+#ifdef DEBUGMEM
+    totalmem -= nbytes;
+#endif
     free(fatso);
 }
 
+#ifdef DEBUGMEM
+#include <stdio.h>
+
+void glob_foo(void *dummy, t_symbol *s, int argc, t_atom *argv)
+{
+    fprintf(stderr, "total mem %d\n", totalmem);
+}
+#endif
