@@ -4,11 +4,15 @@
 
 #if defined(_LANGUAGE_C_PLUS_PLUS) || defined(__cplusplus)
 extern "C" {
-#ifdef NT
-#pragma warning( disable : 4091 ) 
-#endif /* NT */
 #endif
 
+#ifdef NT
+// #pragma warning( disable : 4091 ) 
+#pragma warning( disable : 4305 )  /* uncast const double to float */
+#pragma warning( disable : 4244 )  /* uncast float/int conversion etc. */
+#pragma warning( disable : 4101 )  /* unused automatic variables */
+#endif /* NT */
+
     /* the external storage class is "extern" in UNIX; in NT it's ugly. */
 #ifdef NT
 #ifdef PD_INTERNAL
@@ -113,6 +117,8 @@ typedef enum
     A_CANT
 }  t_atomtype;
 
+#define A_DEFSYMBOL A_DEFSYM	/* better name for this */
+
 typedef struct _atom
 {
     t_atomtype a_type;
@@ -160,7 +166,7 @@ typedef struct _text	    /* a graphical object with at least one textfield */
     t_inlet *te_inlet;
     short te_xpos;
     short te_ypos;
-    short te_width;	/* in length units but should be in chars */
+    short te_width;	/* requested width in chars, 0 if auto */
     unsigned int te_type:2; 	/* from defs below */
 } t_text;
 
@@ -197,6 +203,8 @@ EXTERN t_symbol s_anything;
 EXTERN t_symbol s_signal;
 EXTERN t_symbol s__N;
 EXTERN t_symbol s__X;
+EXTERN t_symbol s_x;
+EXTERN t_symbol s_y;
 EXTERN t_symbol s_;
 
 /* --------- prototypes from the central message system ----------- */
@@ -206,7 +214,7 @@ EXTERN t_symbol *gensym(char *s);
 EXTERN t_gotfn getfn(t_pd *x, t_symbol *s);
 EXTERN t_gotfn zgetfn(t_pd *x, t_symbol *s);
 EXTERN void nullfn(void);
-EXTERN void vmess(t_pd *x, t_symbol *s, char *fmt, ...);
+EXTERN void pd_vmess(t_pd *x, t_symbol *s, char *fmt, ...);
 #define mess0(x, s) ((*getfn((x), (s)))((x)))
 #define mess1(x, s, a) ((*getfn((x), (s)))((x), (a)))
 #define mess2(x, s, a,b) ((*getfn((x), (s)))((x), (a),(b)))
@@ -256,6 +264,7 @@ EXTERN void binbuf_clear(t_binbuf *x);
 EXTERN void binbuf_add(t_binbuf *x, int argc, t_atom *argv);
 EXTERN void binbuf_addv(t_binbuf *x, char *fmt, ...);
 EXTERN void binbuf_addbinbuf(t_binbuf *x, t_binbuf *y);
+EXTERN void binbuf_addsemi(t_binbuf *x);
 EXTERN void binbuf_restore(t_binbuf *x, int argc, t_atom *argv);
 EXTERN void binbuf_print(t_binbuf *x);
 EXTERN int binbuf_getnatom(t_binbuf *x);
@@ -263,6 +272,8 @@ EXTERN t_atom *binbuf_getvec(t_binbuf *x);
 EXTERN void binbuf_eval(t_binbuf *x, t_pd *target, int argc, t_atom *argv);
 EXTERN int binbuf_read(t_binbuf *b, char *filename, char *dirname,
     int crflag);
+EXTERN int binbuf_read_via_path(t_binbuf *b, char *filename, char *dirname,
+    int crflag);
 EXTERN int binbuf_write(t_binbuf *x, char *filename, char *dir,
     int crflag);
 EXTERN void binbuf_evalfile(t_symbol *name, t_symbol *dir);
@@ -273,7 +284,8 @@ EXTERN t_clock *clock_new(void *owner, t_method fn);
 EXTERN void clock_set(t_clock *x, double systime);
 EXTERN void clock_delay(t_clock *x, double delaytime);
 EXTERN void clock_unset(t_clock *x);
-EXTERN double clock_getsystime(void);
+EXTERN double clock_getlogicaltime(void);
+EXTERN double clock_getsystime(void); /* OBSOLETE; use clock_getlogicaltime() */
 EXTERN double clock_gettimesince(double prevsystime);
 EXTERN double clock_getsystimeafter(double delaytime);
 EXTERN void clock_free(t_clock *x);
@@ -334,6 +346,9 @@ EXTERN t_canvas *canvas_getcurrent(void);
 EXTERN void canvas_makefilename(t_canvas *c, char *file,
     char *result,int resultsize);
 EXTERN t_symbol *canvas_getdir(t_canvas *x);
+EXTERN int sys_fontwidth(int fontsize);
+EXTERN int sys_fontheight(int fontsize);
+EXTERN void canvas_dataproperties(t_canvas *x, t_scalar *sc, t_binbuf *b);
 
 /* ---------------- widget behaviors ---------------------- */
 
@@ -375,6 +390,9 @@ EXTERN char *class_getname(t_class *c);
 EXTERN char *class_gethelpname(t_class *c);
 EXTERN void class_setdrawcommand(t_class *c);
 EXTERN int class_isdrawcommand(t_class *c);
+EXTERN void class_domainsignalin(t_class *c, int onset);
+#define CLASS_MAINSIGNALIN(c, type, field) \
+    class_domainsignalin(c, (char *)(&((type *)0)->field) - (char *)0)
 
 #ifndef PD_CLASS_DEF
 #define class_addbang(x, y) class_addbang((x), (t_method)(y))
@@ -492,7 +510,8 @@ EXTERN float garray_get(t_garray *x, t_symbol *s, t_int indx);
 EXTERN void garray_redraw(t_garray *x);
 EXTERN int garray_npoints(t_garray *x);
 EXTERN char *garray_vec(t_garray *x);
-
+EXTERN void garray_resize(t_garray *x, t_floatarg f);
+EXTERN void garray_usedindsp(t_garray *x);
 EXTERN t_class *scalar_class;
 
 /* ------- GUI interface - functions to send strings to TK --------- */
@@ -517,6 +536,7 @@ EXTERN void c_addmess(t_method fn, t_symbol *sel, t_atomtype arg1, ...);
 #define t_freebytes freebytes
 #define t_resizebytes resizebytes
 #define typedmess pd_typedmess
+#define vmess pd_vmess
 
 #if defined(_LANGUAGE_C_PLUS_PLUS) || defined(__cplusplus)
 }
