@@ -1256,8 +1256,8 @@ void sys_poll_midi(void)
     		case 2: inmidi_polyaftertouch(0, chan, byte1, byte2); break; 
     		case 3: inmidi_controlchange(0, chan, byte1, byte2); break; 
     		case 4: inmidi_programchange(0, chan, byte1); break; 
-    		case 6: inmidi_aftertouch(0, chan, byte1); break; 
-    		case 5: inmidi_pitchbend(0, chan, byte1<<7 + byte2 - 8192); break; 
+    		case 5: inmidi_aftertouch(0, chan, byte1); break; 
+    		case 6: inmidi_pitchbend(0, chan, (byte1<<7) + byte2 - 8192); break; 
     	    }
     	    nt_isnextevent = 0;
     	}
