@@ -312,7 +312,7 @@ static void binop2_le_float(t_binop *x, t_float f)
     outlet_float(x->x_obj.ob_outlet, (x->x_f1 = f) <= x->x_f2);
 }
 
-/* ------------- binop3: &, |, &&, ||, %, mod, div ------------------ */
+/* ------------- binop3: &, |, &&, ||, <<, >>, %, mod, div ------------------ */
 
 static void *binop3_new(t_class *fixclass, t_floatarg f)
 {
@@ -400,6 +400,44 @@ static void binop2_lo_float(t_binop *x, t_float f)
     outlet_float(x->x_obj.ob_outlet, ((int)(x->x_f1 = f)) || (int)(x->x_f2));
 }
 
+/* --------------------------- << ---------------------------- */
+
+static t_class *binop3_ls_class;
+
+static void *binop3_ls_new(t_floatarg f)
+{
+    return (binop3_new(binop3_ls_class, f));
+}
+
+static void binop2_ls_bang(t_binop *x)
+{
+    outlet_float(x->x_obj.ob_outlet, ((int)(x->x_f1)) << (int)(x->x_f2));
+}
+
+static void binop2_ls_float(t_binop *x, t_float f)
+{
+    outlet_float(x->x_obj.ob_outlet, ((int)(x->x_f1 = f)) << (int)(x->x_f2));
+}
+
+/* --------------------------- >> ---------------------------- */
+
+static t_class *binop3_rs_class;
+
+static void *binop3_rs_new(t_floatarg f)
+{
+    return (binop3_new(binop3_rs_class, f));
+}
+
+static void binop2_rs_bang(t_binop *x)
+{
+    outlet_float(x->x_obj.ob_outlet, ((int)(x->x_f1)) >> (int)(x->x_f2));
+}
+
+static void binop2_rs_float(t_binop *x, t_float f)
+{
+    outlet_float(x->x_obj.ob_outlet, ((int)(x->x_f1 = f)) >> (int)(x->x_f2));
+}
+
 /* --------------------------- % ---------------------------- */
 
 static t_class *binop3_pc_class;
@@ -602,9 +640,6 @@ static void exp_float(t_object *x, t_float f)
 #endif
     if (f > MAXLOG) f = MAXLOG;
     g = expf(f);
-#ifdef NT
-    sprintf(buf, "", f, g); 	/* compiler bug in MSVC? */
-#endif
     outlet_float(x->ob_outlet, g);
 }
 
@@ -740,6 +775,18 @@ void x_arithmetic_setup(void)
     class_addfloat(binop3_lo_class, (t_method)binop2_lo_float);
     class_sethelpsymbol(binop3_lo_class, binop23_sym);
 
+    binop3_ls_class = class_new(gensym("<<"), (t_newmethod)binop3_ls_new, 0,
+    	sizeof(t_binop), 0, A_DEFFLOAT, 0);
+    class_addbang(binop3_ls_class, binop2_ls_bang);
+    class_addfloat(binop3_ls_class, (t_method)binop2_ls_float);
+    class_sethelpsymbol(binop3_ls_class, binop23_sym);
+
+    binop3_rs_class = class_new(gensym(">>"), (t_newmethod)binop3_rs_new, 0,
+    	sizeof(t_binop), 0, A_DEFFLOAT, 0);
+    class_addbang(binop3_rs_class, binop2_rs_bang);
+    class_addfloat(binop3_rs_class, (t_method)binop2_rs_float);
+    class_sethelpsymbol(binop3_rs_class, binop23_sym);
+
     binop3_pc_class = class_new(gensym("%"), (t_newmethod)binop3_pc_new, 0,
     	sizeof(t_binop), 0, A_DEFFLOAT, 0);
     class_addbang(binop3_pc_class, binop2_pc_bang);
