@@ -291,7 +291,7 @@ static void receive_free(t_receive *x)
 static void receive_setup(void)
 {
     receive_class = class_new(gensym("receive"), (t_newmethod)receive_new, 
-    	(t_method)receive_free, sizeof(t_receive), CLASS_NOINLET, A_SYMBOL, 0);
+    	(t_method)receive_free, sizeof(t_receive), CLASS_NOINLET, A_DEFSYM, 0);
     class_addcreator((t_newmethod)receive_new, gensym("r"), A_DEFSYM, 0);
     class_addbang(receive_class, receive_bang);
     class_addfloat(receive_class, (t_method)receive_float);
@@ -500,34 +500,58 @@ static void route_list(t_route *x, t_symbol *sel, int argc, t_atom *argv)
     	    else outlet_list(e->e_outlet, 0, argc-1, argv+1);
     	    return;
     	}
-    	outlet_list(x->x_rejectout, 0, argc, argv);
     }
-    else    /* symbol given list */
+    else    /* symbol arguments */
     {
-    	for (nelement = x->x_nelement, e = x->x_vec; nelement--; e++)
+	if (argc > 1)	    /* 2 or more args: treat as "list" */
 	{
-    	    if (e->e_w.w_symbol == &s_list)
-    	    {
-    		if (argc > 0 && argv[0].a_type == A_SYMBOL)
-    	    	    outlet_anything(e->e_outlet, argv[0].a_w.w_symbol,
-    	    		argc-1, argv+1);
-    		else if (argc > 1) outlet_list(e->e_outlet, 0, argc, argv);
-    		return;
-    	    }
-    	    else if (e->e_w.w_symbol == &s_float && argc == 1 &&
-	    	argv[0].a_type  == A_FLOAT)
-    	    {
-    		outlet_float(e->e_outlet, argv[0].a_w.w_float);
-    		return;
-    	    }
-    	    else if (e->e_w.w_symbol == &s_bang && argc == 0)
-    	    {
-    		outlet_bang(e->e_outlet);
-    		return;
-    	    }
+	    for (nelement = x->x_nelement, e = x->x_vec; nelement--; e++)
+	    {
+	    	if (e->e_w.w_symbol == &s_list)
+    		{
+    		    if (argc > 0 && argv[0].a_type == A_SYMBOL)
+    	    		outlet_anything(e->e_outlet, argv[0].a_w.w_symbol,
+    	    		    argc-1, argv+1);
+    		    else outlet_list(e->e_outlet, 0, argc, argv);
+    		    return;
+	    	}
+	    }
+	}
+	else if (argc == 0)	    /* no args: treat as "bang" */
+	{
+	    for (nelement = x->x_nelement, e = x->x_vec; nelement--; e++)
+	    {
+	    	if (e->e_w.w_symbol == &s_bang)
+		{
+		    outlet_bang(e->e_outlet);
+		    return;
+	    	}
+	    }
+	}
+	else if (argv[0].a_type == A_FLOAT) 	/* one float arg */
+	{
+	    for (nelement = x->x_nelement, e = x->x_vec; nelement--; e++)
+	    {
+	    	if (e->e_w.w_symbol == &s_float)
+		{
+		    outlet_float(e->e_outlet, argv[0].a_w.w_float);
+		    return;
+	    	}
+	    }
+	}
+	else
+	{
+	    for (nelement = x->x_nelement, e = x->x_vec; nelement--; e++)
+	    {
+	    	if (e->e_w.w_symbol == &s_symbol)
+		{
+		    outlet_symbol(e->e_outlet, argv[0].a_w.w_symbol);
+		    return;
+	    	}
+	    }
 	}
-    	outlet_list(x->x_rejectout, 0, argc, argv);
     }
+    outlet_list(x->x_rejectout, 0, argc, argv);
 }
 
 
@@ -548,12 +572,13 @@ static void *route_new(t_symbol *s, int argc, t_atom *argv)
     	SETFLOAT(&a, 0);
     	argv = &a;
     }
+    x->x_type = argv[0].a_type;
     x->x_nelement = argc;
     x->x_vec = (t_routeelement *)getbytes(argc * sizeof(*x->x_vec));
     for (n = 0, e = x->x_vec; n < argc; n++, e++)
     {
     	e->e_outlet = outlet_new(&x->x_obj, &s_list);
-    	if ((x->x_type = argv->a_type) == A_FLOAT)
+    	if (x->x_type == A_FLOAT)
     	    e->e_w.w_float = atom_getfloatarg(n, argc, argv);
 	else e->e_w.w_symbol = atom_getsymbolarg(n, argc, argv);
     }
@@ -879,7 +904,11 @@ static void *trigger_new(t_symbol *s, int argc, t_atom *argv)
     	    u->u_type = A_GIMME, u->u_outlet = outlet_new(&x->x_obj, &s_list);
     	else if (c == 's')
     	    u->u_type = A_SYMBOL, u->u_outlet = outlet_new(&x->x_obj, &s_symbol);
-    	else pd_error(x, "trigger: %s: bad type", ap->a_w.w_symbol->s_name);
+    	else
+	{
+	    pd_error(x, "trigger: %s: bad type", ap->a_w.w_symbol->s_name);
+    	    u->u_type = A_FLOAT, u->u_outlet = outlet_new(&x->x_obj, &s_float);
+    	}
     }
     return (x);
 }
@@ -1015,7 +1044,8 @@ static void spigot_setup(void)
     class_addanything(spigot_class, spigot_anything);
 }
 
-static t_class *moses_class;   	/* ----------- moses --------------- */
+/* --------------------------- moses ----------------------------- */
+static t_class *moses_class;
 
 typedef struct _moses
 {
@@ -1228,6 +1258,74 @@ void change_setup(void)
     	A_DEFFLOAT, 0);
 }
 
+/* -------------------- value ------------------------------ */
+
+static t_class *value_class, *vcommon_class;
+
+typedef struct vcommon
+{
+    t_pd c_pd;
+    int c_refcount;
+    t_float c_f;
+} t_vcommon;
+
+typedef struct _value
+{
+    t_object x_obj;
+    t_symbol *x_sym;
+    t_vcommon *x_vcommon;
+} t_value;
+
+static void *value_new(t_symbol *s)
+{
+    t_value *x = (t_value *)pd_new(value_class);
+    t_vcommon *c = (t_vcommon *)pd_findbyclass(s, vcommon_class);
+    if (!c)
+    {
+    	c = (t_vcommon *)pd_new(vcommon_class);
+	c->c_f = 0;
+	c->c_refcount = 0;
+	pd_bind(&c->c_pd, s);
+    }
+    c->c_refcount++;
+    x->x_sym = s;
+    x->x_vcommon = c;
+    outlet_new(&x->x_obj, &s_float);
+    return (x);
+}
+
+static void value_bang(t_value *x)
+{
+    outlet_float(x->x_obj.ob_outlet, x->x_vcommon->c_f);
+}
+
+static void value_float(t_value *x, t_float f)
+{
+    x->x_vcommon->c_f = f;
+}
+
+static void value_ff(t_value *x)
+{
+    t_vcommon *c = x->x_vcommon;
+    if (!--c->c_refcount)
+    {
+    	pd_unbind(&c->c_pd, x->x_sym);
+	pd_free(&c->c_pd);
+    }
+}
+
+static void value_setup(void)
+{
+    value_class = class_new(gensym("value"), (t_newmethod)value_new,
+    	(t_method)value_ff,
+    	sizeof(t_value), 0, A_DEFSYM, 0);
+    class_addcreator((t_newmethod)value_new, gensym("v"), A_DEFSYM, 0);
+    class_addbang(value_class, value_bang);
+    class_addfloat(value_class, value_float);
+    vcommon_class = class_new(gensym("value"), 0, 0,
+    	sizeof(t_vcommon), CLASS_PD, 0);
+}
+
 /* -------------- overall setup routine for this file ----------------- */
 
 void x_connective_setup(void)
@@ -1249,4 +1347,5 @@ void x_connective_setup(void)
     makefilename_setup();
     swap_setup();
     change_setup();
+    value_setup();
 }
