@@ -12,6 +12,99 @@ void outmidi_pitchbend(int portno, int channel, int value);
 void outmidi_aftertouch(int portno, int channel, int value);
 void outmidi_polyaftertouch(int portno, int channel, int pitch, int value);
 
+/* ----------------------- midiin and sysexin ------------------------- */
+
+static t_symbol *midiin_sym, *sysexin_sym;
+
+static t_class *midiin_class, *sysexin_class;
+
+typedef struct _midiin
+{
+    t_object x_obj;
+    t_outlet *x_outlet1;
+    t_outlet *x_outlet2;
+} t_midiin;
+
+static void *midiin_new( void)
+{
+    t_midiin *x = (t_midiin *)pd_new(midiin_class);
+    x->x_outlet1 = outlet_new(&x->x_obj, &s_float);
+    x->x_outlet2 = outlet_new(&x->x_obj, &s_float);
+    pd_bind(&x->x_obj.ob_pd, midiin_sym);
+#ifndef __linux__
+    pd_error(x, "midiin: works under Linux only");
+#endif
+    return (x);
+}
+
+static void midiin_list(t_midiin *x, t_symbol *s, int ac, t_atom *av)
+{
+    outlet_float(x->x_outlet2, atom_getfloatarg(1, ac, av) + 1);
+    outlet_float(x->x_outlet1, atom_getfloatarg(0, ac, av));
+}
+
+static void midiin_free(t_midiin *x)
+{
+    pd_unbind(&x->x_obj.ob_pd, midiin_sym);
+}
+
+static void *sysexin_new( void)
+{
+    t_midiin *x = (t_midiin *)pd_new(sysexin_class);
+    x->x_outlet1 = outlet_new(&x->x_obj, &s_float);
+    x->x_outlet2 = outlet_new(&x->x_obj, &s_float);
+    pd_bind(&x->x_obj.ob_pd, sysexin_sym);
+#ifndef __linux__
+    pd_error(x, "sysexin: works under Linux only");
+#endif
+    return (x);
+}
+
+static void sysexin_free(t_midiin *x)
+{
+    pd_unbind(&x->x_obj.ob_pd, sysexin_sym);
+}
+
+static void midiin_setup(void)
+{
+    midiin_class = class_new(gensym("midiin"), (t_newmethod)midiin_new,
+    	(t_method)midiin_free, sizeof(t_midiin),
+	    CLASS_NOINLET, A_DEFFLOAT, 0);
+    class_addlist(midiin_class, midiin_list);
+    class_sethelpsymbol(midiin_class, gensym("midi"));
+    midiin_sym = gensym("#midiin");
+
+    sysexin_class = class_new(gensym("sysexin"), (t_newmethod)sysexin_new,
+    	(t_method)sysexin_free, sizeof(t_midiin),
+	    CLASS_NOINLET, A_DEFFLOAT, 0);
+    class_addlist(sysexin_class, midiin_list);
+    class_sethelpsymbol(sysexin_class, gensym("midi"));
+    sysexin_sym = gensym("#sysexin");
+}
+
+void inmidi_byte(int portno, int byte)
+{
+    static int sysex;
+    t_atom at[2];
+    if (byte == 0xf0)
+    	sysex |= (1 << portno);
+    if (sysexin_sym->s_thing && (sysex & (1 << portno)))
+    {
+    	SETFLOAT(at, byte);
+    	SETFLOAT(at+1, portno);
+    	pd_list(sysexin_sym->s_thing, 0, 2, at);
+    }
+    if (byte == 0xf7)
+    	sysex &= (~(1 << portno));
+
+    if (midiin_sym->s_thing)
+    {
+    	SETFLOAT(at, byte);
+    	SETFLOAT(at+1, portno);
+    	pd_list(midiin_sym->s_thing, 0, 2, at);
+    }
+}
+
 /* ----------------------- notein ------------------------- */
 
 static t_symbol *notein_sym;
@@ -78,7 +171,7 @@ void inmidi_noteon(int portno, int channel, int pitch, int velo)
     	t_atom at[3];
     	SETFLOAT(at, pitch);
     	SETFLOAT(at+1, velo);
-    	SETFLOAT(at+2, (channel + (portno << 4)));
+    	SETFLOAT(at+2, (channel + (portno << 4) + 1));
     	pd_list(notein_sym->s_thing, &s_list, 3, at);
     }
 }
@@ -152,7 +245,7 @@ void inmidi_controlchange(int portno, int channel, int ctlnumber, int value)
     	t_atom at[3];
     	SETFLOAT(at, ctlnumber);
     	SETFLOAT(at+1, value);
-    	SETFLOAT(at+2, (channel + (portno << 4)));
+    	SETFLOAT(at+2, (channel + (portno << 4) + 1));
     	pd_list(ctlin_sym->s_thing, &s_list, 3, at);
     }
 }
@@ -217,8 +310,8 @@ void inmidi_programchange(int portno, int channel, int value)
     if (pgmin_sym->s_thing)
     {
     	t_atom at[2];
-    	SETFLOAT(at, value);
-    	SETFLOAT(at+1, (channel + (portno << 4)));
+    	SETFLOAT(at, value + 1);
+    	SETFLOAT(at+1, (channel + (portno << 4) + 1));
     	pd_list(pgmin_sym->s_thing, &s_list, 2, at);
     }
 }
@@ -283,7 +376,7 @@ void inmidi_pitchbend(int portno, int channel, int value)
     {
     	t_atom at[2];
     	SETFLOAT(at, value);
-    	SETFLOAT(at+1, (channel + (portno << 4)));
+    	SETFLOAT(at+1, (channel + (portno << 4) + 1));
     	pd_list(bendin_sym->s_thing, &s_list, 2, at);
     }
 }
@@ -349,7 +442,7 @@ void inmidi_aftertouch(int portno, int channel, int value)
     {
     	t_atom at[2];
     	SETFLOAT(at, value);
-    	SETFLOAT(at+1, (channel + (portno << 4)));
+    	SETFLOAT(at+1, (channel + (portno << 4) + 1));
     	pd_list(touchin_sym->s_thing, &s_list, 2, at);
     }
 }
@@ -421,11 +514,52 @@ void inmidi_polyaftertouch(int portno, int channel, int pitch, int value)
     	t_atom at[3];
     	SETFLOAT(at, pitch);
     	SETFLOAT(at+1, value);
-    	SETFLOAT(at+2, (channel + (portno << 4)));
+    	SETFLOAT(at+2, (channel + (portno << 4) + 1));
     	pd_list(polytouchin_sym->s_thing, &s_list, 3, at);
     }
 }
 
+/* -------------------------- midiout -------------------------- */
+
+static t_class *midiout_class;
+
+#ifdef __linux__
+void sys_putmidibyte(int portno, int byte);
+#endif
+
+typedef struct _midiout
+{
+    t_object x_obj;
+    t_float x_portno;
+} t_midiout;
+
+static void *midiout_new(t_floatarg portno)
+{
+    t_midiout *x = (t_midiout *)pd_new(midiout_class);
+    if (portno <= 0) portno = 1;
+    x->x_portno = portno;
+    floatinlet_new(&x->x_obj, &x->x_portno);
+#ifndef __linux__
+    pd_error(x, "midiout: works under Linux only");
+#endif
+    return (x);
+}
+
+static void midiout_float(t_midiout *x, t_floatarg f)
+{
+#ifdef __linux__
+    sys_putmidibyte(x->x_portno - 1, f);
+#endif
+}
+
+static void midiout_setup(void)
+{
+    midiout_class = class_new(gensym("midiout"), (t_newmethod)midiout_new, 0,
+    	sizeof(t_midiout), 0, A_DEFFLOAT, 0);
+    class_addfloat(midiout_class, midiout_float);
+    class_sethelpsymbol(midiout_class, gensym("midi"));
+}
+
 /* -------------------------- noteout -------------------------- */
 
 static t_class *noteout_class;
@@ -451,6 +585,8 @@ static void *noteout_new(t_floatarg channel)
 static void noteout_float(t_noteout *x, t_float f)
 {
     int binchan = x->x_channel - 1;
+    if (binchan < 0)
+    	binchan = 0;
     outmidi_noteon((binchan >> 4),
     	(binchan & 15), (int)f, (int)x->x_velo);
 }
@@ -489,6 +625,8 @@ static void *ctlout_new(t_floatarg ctl, t_floatarg channel)
 static void ctlout_float(t_ctlout *x, t_float f)
 {
     int binchan = x->x_channel - 1;
+    if (binchan < 0)
+    	binchan = 0;
     outmidi_controlchange((binchan >> 4),
     	(binchan & 15), (int)(x->x_ctl), (int)f);
 }
@@ -525,6 +663,8 @@ static void pgmout_float(t_pgmout *x, t_floatarg f)
 {
     int binchan = x->x_channel - 1;
     int n = f - 1;
+    if (binchan < 0)
+    	binchan = 0;
     if (n < 0) n = 0;
     else if (n > 127) n = 127;
     outmidi_programchange((binchan >> 4),
@@ -563,6 +703,8 @@ static void bendout_float(t_bendout *x, t_float f)
 {
     int binchan = x->x_channel - 1;
     int n = (int)f +  8192;
+    if (binchan < 0)
+    	binchan = 0;
     outmidi_pitchbend((binchan >> 4), (binchan & 15), n);
 }
 
@@ -596,6 +738,8 @@ static void *touchout_new(t_floatarg channel)
 static void touchout_float(t_touchout *x, t_float f)
 {
     int binchan = x->x_channel - 1;
+    if (binchan < 0)
+    	binchan = 0;
     outmidi_aftertouch((binchan >> 4), (binchan & 15), (int)f);
 }
 
@@ -632,6 +776,8 @@ static void *polytouchout_new(t_floatarg channel)
 static void polytouchout_float(t_polytouchout *x, t_float n)
 {
     int binchan = x->x_channel - 1;
+    if (binchan < 0)
+    	binchan = 0;
     outmidi_polyaftertouch((binchan >> 4), (binchan & 15), x->x_pitch, n);
 }
 
@@ -1015,12 +1161,14 @@ static void bag_setup(void)
 
 void x_midi_setup(void)
 {
+    midiin_setup();
     notein_setup();
     ctlin_setup();
     pgmin_setup();
     bendin_setup();
     touchin_setup();
     polytouchin_setup();
+    midiout_setup();
     noteout_setup();
     ctlout_setup();
     pgmout_setup();
